@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -19,12 +19,13 @@
  **/
  
 #include "Memoizer.h"
-#include <cstddef>
+
+#include <unordered_map>
 
 namespace love
 {
 
-std::map<void *, void *> Memoizer::objectMap;
+static std::unordered_map<void *, void *> objectMap;
 
 void Memoizer::add(void *key, void *val)
 {
