@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2016 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -34,9 +34,12 @@ class Module : public Object
 {
 public:
 
+	static love::Type type;
+
 	enum ModuleType
 	{
 		M_AUDIO,
+		M_DATA,
 		M_EVENT,
 		M_FILESYSTEM,
 		M_FONT,
@@ -52,11 +55,12 @@ public:
 		M_THREAD,
 		M_TIMER,
 		M_TOUCH,
-		M_WINDOW,
 		M_VIDEO,
+		M_WINDOW,
 		M_MAX_ENUM
 	};
 
+	Module();
 	virtual ~Module();
 
     /**
