@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2015 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -61,22 +61,18 @@
 namespace love
 {
 
-struct Vertex
+struct Rect
 {
-	float x, y;
-	float s, t;
-	unsigned char r, g, b, a;
+	int x, y;
+	int w, h;
+
+	bool operator == (const Rect &rhs) const
+	{
+		return x == rhs.x && y == rhs.y && w == rhs.w && h == rhs.h;
+	}
 };
 
-struct Triangle
-{
-	Triangle(const Vertex &x, const Vertex &y, const Vertex &z)
-		: a(x), b(y), c(z)
-	{}
-	Vertex a, b, c;
-};
-
-inline int next_p2(int x)
+inline int nextP2(int x)
 {
 	x += (x == 0);
 	x--;
@@ -84,9 +80,9 @@ inline int next_p2(int x)
 	return ++x;
 }
 
-inline float next_p2(float x)
+inline float nextP2(float x)
 {
-	return static_cast<float>(next_p2(static_cast<int>(x)));
+	return (float) nextP2((int) x);
 }
 
 } // love
