@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -24,19 +24,27 @@
 // STD
 #include <queue>
 #include <map>
-#include <iostream>
+#include <vector>
 #include <cmath>
 
 // LOVE
 #include "common/config.h"
 #include "common/Exception.h"
 #include "thread/threads.h"
+#include "audio/Source.h"
 
 // OpenAL
-#ifdef LOVE_MACOSX_USE_FRAMEWORKS
+#ifdef LOVE_APPLE_USE_FRAMEWORKS
+#ifdef LOVE_IOS
+#include <OpenAL/alc.h>
+#include <OpenAL/al.h>
+#include <OpenAL/oalMacOSX_OALExtensions.h>
+#include <OpenAL/oalStaticBufferExtension.h>
+#else
 #include <OpenAL-Soft/alc.h>
 #include <OpenAL-Soft/al.h>
 #include <OpenAL-Soft/alext.h>
+#endif
 #else
 #include <AL/alc.h>
 #include <AL/al.h>
@@ -72,34 +80,23 @@ public:
 
 	void update();
 
-	int getSourceCount() const;
+	int getActiveSourceCount() const;
 	int getMaxSources() const;
 
-	bool play(Source *source, ALuint &out);
-	void stop();
-	void stop(Source *source);
-	void pause();
-	void pause(Source *source);
-	void resume();
-	void resume(Source *source);
-	void rewind();
-	void rewind(Source *source);
-	void softRewind(Source *source);
-	void seek(Source *source, float offset, void *unit);
-	float tell(Source *source, void *unit);
-
 private:
 
+	friend class Source;
+	LOVE_WARN_UNUSED thread::Lock lock();
+	std::vector<love::audio::Source*> getPlayingSources();
+
 	/**
 	 * Makes the specified OpenAL source available for use.
 	 * @param source The OpenAL source.
 	 **/
-	void release(Source *source);
-
-	ALuint findi(const Source *source) const;
+	bool releaseSource(Source *source, bool stop = true);
 
+	bool assignSource(Source *source, ALuint &out, char &wasPlaying);
 	bool findSource(Source *source, ALuint &out);
-	bool removeSource(Source *source);
 
 	// Maximum possible number of OpenAL sources the pool attempts to generate.
 	static const int MAX_SOURCES = 64;
@@ -118,7 +115,7 @@ private:
 
 	// Only one thread can access this object at the same time. This mutex will
 	// make sure of that.
-	thread::Mutex *mutex;
+	love::thread::MutexRef mutex;
 
 }; // Pool
 
