@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -25,13 +25,28 @@
 #include "common/config.h"
 #include "common/Object.h"
 #include "audio/Source.h"
+#include "audio/Filter.h"
 #include "sound/SoundData.h"
 #include "sound/Decoder.h"
+#include "Audio.h"
+#include "Filter.h"
+
+// STL
+#include <vector>
+#include <stack>
+
+// C
+#include <float.h>
 
 // OpenAL
-#ifdef LOVE_MACOSX_USE_FRAMEWORKS
+#ifdef LOVE_APPLE_USE_FRAMEWORKS
+#ifdef LOVE_IOS
+#include <OpenAL/alc.h>
+#include <OpenAL/al.h>
+#else
 #include <OpenAL-Soft/alc.h>
 #include <OpenAL-Soft/al.h>
+#endif
 #else
 #include <AL/alc.h>
 #include <AL/al.h>
@@ -44,6 +59,13 @@ namespace audio
 namespace openal
 {
 
+#ifdef LOVE_IOS
+// OpenAL on iOS barfs if the max distance is +inf.
+static const float MAX_ATTENUATION_DISTANCE = 1000000.0f;
+#else
+static const float MAX_ATTENUATION_DISTANCE = FLT_MAX;
+#endif
+
 class Audio;
 class Pool;
 
@@ -60,9 +82,15 @@ public:
 		return buffer;
 	}
 
+	inline ALsizei getSize() const
+	{
+		return size;
+	}
+
 private:
 
 	ALuint buffer;
+	ALsizei size;
 
 }; // StaticDataBuffer
 
@@ -72,6 +100,7 @@ public:
 
 	Source(Pool *pool, love::sound::SoundData *soundData);
 	Source(Pool *pool, love::sound::Decoder *decoder);
+	Source(Pool *pool, int sampleRate, int bitDepth, int channels, int buffers);
 	Source(const Source &s);
 	virtual ~Source();
 
@@ -79,33 +108,28 @@ public:
 	virtual bool play();
 	virtual void stop();
 	virtual void pause();
-	virtual void resume();
-	virtual void rewind();
-	virtual bool isStopped() const;
-	virtual bool isPaused() const;
+	virtual bool isPlaying() const;
 	virtual bool isFinished() const;
 	virtual bool update();
 	virtual void setPitch(float pitch);
 	virtual float getPitch() const;
 	virtual void setVolume(float volume);
 	virtual float getVolume() const;
-	virtual void seekAtomic(float offset, void *unit);
 	virtual void seek(float offset, Unit unit);
-	virtual float tellAtomic(void *unit) const;
 	virtual float tell(Unit unit);
+	virtual double getDuration(Unit unit);
 	virtual void setPosition(float *v);
 	virtual void getPosition(float *v) const;
 	virtual void setVelocity(float *v);
 	virtual void getVelocity(float *v) const;
 	virtual void setDirection(float *v);
 	virtual void getDirection(float *v) const;
-	virtual void setCone(float innerAngle, float outerAngle, float outerVolume);
-	virtual void getCone(float &innerAngle, float &outerAngle, float &outerVolume) const;
+	virtual void setCone(float innerAngle, float outerAngle, float outerVolume, float outerHighGain);
+	virtual void getCone(float &innerAngle, float &outerAngle, float &outerVolume, float &outerHighGain) const;
 	virtual void setRelative(bool enable);
 	virtual bool isRelative() const;
 	void setLooping(bool looping);
 	bool isLooping() const;
-	bool isStatic() const;
 	virtual void setMinVolume(float volume);
 	virtual float getMinVolume() const;
 	virtual void setMaxVolume(float volume);
@@ -116,13 +140,37 @@ public:
 	virtual float getRolloffFactor() const;
 	virtual void setMaxDistance(float distance);
 	virtual float getMaxDistance() const;
-	virtual int getChannels() const;
+	virtual void setAirAbsorptionFactor(float factor);
+	virtual float getAirAbsorptionFactor() const;
+	virtual int getChannelCount() const;
 
-	bool playAtomic();
+	virtual bool setFilter(const std::map<Filter::Parameter, float> &params);
+	virtual bool setFilter();
+	virtual bool getFilter(std::map<Filter::Parameter, float> &params);
+
+	virtual bool setEffect(const char *effect);
+	virtual bool setEffect(const char *effect, const std::map<Filter::Parameter, float> &params);
+	virtual bool unsetEffect(const char *effect);
+	virtual bool getEffect(const char *effect, std::map<Filter::Parameter, float> &params);
+	virtual bool getActiveEffects(std::vector<std::string> &list) const;
+
+	virtual int getFreeBufferCount() const;
+	virtual bool queue(void *data, size_t length, int dataSampleRate, int dataBitDepth, int dataChannels);
+
+	void prepareAtomic();
+	void teardownAtomic();
+
+	bool playAtomic(ALuint source);
 	void stopAtomic();
 	void pauseAtomic();
 	void resumeAtomic();
-	void rewindAtomic();
+
+	static bool play(const std::vector<love::audio::Source*> &sources);
+	static void stop(const std::vector<love::audio::Source*> &sources);
+	static void pause(const std::vector<love::audio::Source*> &sources);
+
+	static std::vector<love::audio::Source*> pause(Pool *pool);
+	static void stop(Pool *pool);
 
 private:
 
@@ -130,62 +178,63 @@ private:
 
 	void setFloatv(float *dst, const float *src) const;
 
-	/**
-	 * Gets the OpenAL format identifier based on number of
-	 * channels and bits.
-	 * @param channels Either 1 (mono) or 2 (stereo).
-	 * @param bitDepth Either 8-bit samples, or 16-bit samples.
-	 * @return One of AL_FORMAT_*, or 0 if unsupported format.
-	 **/
-	ALenum getFormat(int channels, int bitDepth) const;
-
 	int streamAtomic(ALuint buffer, love::sound::Decoder *d);
 
-	Pool *pool;
-	ALuint source;
-	bool valid;
+	Pool *pool = nullptr;
+	ALuint source = 0;
+	bool valid = false;
 
-	static const unsigned int MAX_BUFFERS = 32;
-	ALuint streamBuffers[MAX_BUFFERS];
+	const static int DEFAULT_BUFFERS = 8;
+	const static int MAX_BUFFERS = 64;
+	std::queue<ALuint> streamBuffers;
+	std::stack<ALuint> unusedBuffers;
 
-	StaticDataBuffer *staticBuffer;
+	StrongRef<StaticDataBuffer> staticBuffer;
 
-	float pitch;
-	float volume;
+	float pitch = 1.0f;
+	float volume = 1.0f;
 	float position[3];
 	float velocity[3];
 	float direction[3];
-	bool relative;
-	bool looping;
-	bool paused;
-	float minVolume;
-	float maxVolume;
-	float referenceDistance;
-	float rolloffFactor;
-	float maxDistance;
+	bool relative = false;
+	bool looping = false;
+	float minVolume = 0.0f;
+	float maxVolume = 1.0f;
+	float referenceDistance = 1.0f;
+	float rolloffFactor = 1.0f;
+	float absorptionFactor = 0.0f;
+	float maxDistance = MAX_ATTENUATION_DISTANCE;
 
 	struct Cone
 	{
-		int innerAngle; // degrees
-		int outerAngle; // degrees
-		float outerVolume;
-
-		Cone()
-			: innerAngle(360)
-			, outerAngle(360)
-			, outerVolume(0.0f)
-		{}
+		int innerAngle = 360; // degrees
+		int outerAngle = 360; // degrees
+		float outerVolume = 0.0f;
+		float outerHighGain = 1.0f;
 	} cone;
 
-	float offsetSamples;
-	float offsetSeconds;
+	float offsetSamples = 0.0f;
+	float offsetSeconds = 0.0f;
 
-	int channels;
+	int sampleRate = 0;
+	int channels = 0;
+	int bitDepth = 0;
 
-	love::sound::Decoder *decoder;
+	StrongRef<love::sound::Decoder> decoder;
 
-	unsigned int toLoop;
+	unsigned int toLoop = 0;
+	ALsizei bufferedBytes = 0;
+	int buffers = 0;
 
+	Filter *directfilter = nullptr;
+
+	struct EffectMapStorage
+	{
+		Filter *filter;
+		ALuint slot, target;
+	};
+	std::map<std::string, EffectMapStorage> effectmap;
+	std::stack<ALuint> slotlist;
 }; // Source
 
 } // openal
