@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2016 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -44,6 +44,8 @@ class File : public Object
 {
 public:
 
+	static love::Type type;
+
 	/**
 	 * File open mode.
 	 **/
@@ -202,9 +204,11 @@ public:
 
 	static bool getConstant(const char *in, Mode &out);
 	static bool getConstant(Mode in, const char *&out);
+	static std::vector<std::string> getConstants(Mode);
 
 	static bool getConstant(const char *in, BufferMode &out);
 	static bool getConstant(BufferMode in, const char *&out);
+	static std::vector<std::string> getConstants(BufferMode);
 
 private:
 
