@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -29,13 +29,49 @@ namespace love
 namespace filesystem
 {
 
+love::Type FileData::type("FileData", &Data::type);
+
 FileData::FileData(uint64 size, const std::string &filename)
-	: data(new char[(size_t) size])
+	: data(nullptr)
 	, size((size_t) size)
 	, filename(filename)
 {
-	if (filename.rfind('.') != std::string::npos)
-		extension = filename.substr(filename.rfind('.')+1);
+	try
+	{
+		data = new char[(size_t) size];
+	}
+	catch (std::bad_alloc &)
+	{
+		throw love::Exception("Out of memory.");
+	}
+
+	size_t dotpos = filename.rfind('.');
+
+	if (dotpos != std::string::npos)
+	{
+		extension = filename.substr(dotpos + 1);
+		name = filename.substr(0, dotpos);
+	}
+	else
+		name = filename;
+}
+
+FileData::FileData(const FileData &c)
+	: data(nullptr)
+	, size(c.size)
+	, filename(c.filename)
+	, extension(c.extension)
+	, name(c.name)
+{
+	try
+	{
+		data = new char[(size_t) size];
+	}
+	catch (std::bad_alloc &)
+	{
+		throw love::Exception("Out of memory.");
+	}
+	memcpy(data, c.data, size);
 }
 
 FileData::~FileData()
@@ -43,9 +79,14 @@ FileData::~FileData()
 	delete [] data;
 }
 
+FileData *FileData::clone() const
+{
+	return new FileData(*this);
+}
+
 void *FileData::getData() const
 {
-	return (void *)data;
+	return data;
 }
 
 size_t FileData::getSize() const
@@ -64,23 +105,10 @@ const std::string &FileData::getExtension() const
 	return extension;
 }
 
-bool FileData::getConstant(const char *in, Decoder &out)
+const std::string &FileData::getName() const
 {
-	return decoders.find(in, out);
+	return name;
 }
 
-bool FileData::getConstant(Decoder in, const char  *&out)
-{
-	return decoders.find(in, out);
-}
-
-StringMap<FileData::Decoder, FileData::DECODE_MAX_ENUM>::Entry FileData::decoderEntries[] =
-{
-	{"file", FileData::FILE},
-	{"base64", FileData::BASE64},
-};
-
-StringMap<FileData::Decoder, FileData::DECODE_MAX_ENUM> FileData::decoders(FileData::decoderEntries, sizeof(FileData::decoderEntries));
-
 } // filesystem
 } // love
