@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -18,14 +18,40 @@
  * 3. This notice may not be removed or altered from any source distribution.
  **/
 
-#include "common/config.h"
-
 #include <iostream>
+#include <sstream>
+#include <algorithm>
 
 #include "common/utf8.h"
 #include "common/b64.h"
 
 #include "Filesystem.h"
+#include "File.h"
+
+// PhysFS
+#include "libraries/physfs/physfs.h"
+
+// For great CWD. (Current Working Directory)
+// Using this instead of boost::filesystem which totally
+// cramped our style.
+#ifdef LOVE_WINDOWS
+#	include <windows.h>
+#	include <direct.h>
+#else
+#	include <sys/param.h>
+#	include <unistd.h>
+#endif
+
+#ifdef LOVE_IOS
+#	include "common/ios.h"
+#endif
+
+#include <string>
+
+#ifdef LOVE_ANDROID
+#include <SDL.h>
+#include "common/android.h"
+#endif
 
 namespace
 {
@@ -47,6 +73,22 @@ namespace
 	{
 		return input.substr(getDriveDelim(input)+1);
 	}
+
+	std::string normalize(const std::string &input)
+	{
+		std::stringstream out;
+		bool seenSep = false, isSep = false;
+		for (size_t i = 0; i < input.size(); ++i)
+		{
+			isSep = (input[i] == LOVE_PATH_SEPARATOR[0]);
+			if (!isSep || !seenSep)
+				out << input[i];
+			seenSep = isSep;
+		}
+
+		return out.str();
+	}
+
 }
 
 namespace love
@@ -57,15 +99,16 @@ namespace physfs
 {
 
 Filesystem::Filesystem()
-	: initialized(false)
-	, fused(false)
+	: fused(false)
 	, fusedSet(false)
 {
+	requirePath = {"?.lua", "?/init.lua"};
+	cRequirePath = {"??"};
 }
 
 Filesystem::~Filesystem()
 {
-	if (initialized)
+	if (PHYSFS_isInit())
 		PHYSFS_deinit();
 }
 
@@ -77,8 +120,10 @@ const char *Filesystem::getName() const
 void Filesystem::init(const char *arg0)
 {
 	if (!PHYSFS_init(arg0))
-		throw Exception(PHYSFS_getLastError());
-	initialized = true;
+		throw love::Exception("Failed to initialize filesystem: %s", PHYSFS_getErrorByCode(PHYSFS_getLastErrorCode()));
+
+	// Enable symlinks by default.
+	setSymlinksEnabled(true);
 }
 
 void Filesystem::setFused(bool fused)
@@ -96,9 +141,9 @@ bool Filesystem::isFused() const
 	return fused;
 }
 
-bool Filesystem::setIdentity(const char *ident, bool appendToPath)
+bool Filesystem::setIdentity(const char *ident, bool appendToPath) 
 {
-	if (!initialized)
+	if (!PHYSFS_isInit())
 		return false;
 
 	std::string old_save_path = save_path_full;
@@ -116,6 +161,27 @@ bool Filesystem::setIdentity(const char *ident, bool appendToPath)
 	else
 		save_path_full += save_path_relative;
 
+	save_path_full = normalize(save_path_full);	
+	
+#ifdef LOVE_ANDROID
+	if (save_identity == "")
+		save_identity = "unnamed";
+
+	std::string storage_path;
+	if (isAndroidSaveExternal())
+		storage_path = SDL_AndroidGetExternalStoragePath();
+	else
+		storage_path = SDL_AndroidGetInternalStoragePath();
+
+	std::string save_directory = storage_path + "/save";
+
+	save_path_full = storage_path + std::string("/save/") + save_identity;
+
+	if (!love::android::directoryExists(save_path_full.c_str()) &&
+			!love::android::mkdir(save_path_full.c_str()))
+		SDL_Log("Error: Could not create save directory %s!", save_path_full.c_str());
+#endif
+
 	// We now have something like:
 	// save_identity: game
 	// save_path_relative: ./LOVE/game
@@ -124,11 +190,16 @@ bool Filesystem::setIdentity(const char *ident, bool appendToPath)
 	// We don't want old read-only save paths to accumulate when we set a new
 	// identity.
 	if (!old_save_path.empty())
-		PHYSFS_removeFromSearchPath(old_save_path.c_str());
+		PHYSFS_unmount(old_save_path.c_str());
 
 	// Try to add the save directory to the search path.
 	// (No error on fail, it means that the path doesn't exist).
-	PHYSFS_addToSearchPath(save_path_full.c_str(), appendToPath);
+	PHYSFS_mount(save_path_full.c_str(), nullptr, appendToPath);
+
+	// HACK: This forces setupWriteDirectory to be called the next time a file
+	// is opened for writing - otherwise it won't be called at all if it was
+	// already called at least once before.
+	PHYSFS_setWriteDir(nullptr);
 
 	return true;
 }
@@ -140,19 +211,53 @@ const char *Filesystem::getIdentity() const
 
 bool Filesystem::setSource(const char *source)
 {
-	if (!initialized)
+	if (!PHYSFS_isInit())
 		return false;
 
 	// Check whether directory is already set.
 	if (!game_source.empty())
 		return false;
 
+	std::string new_search_path = source;
+
+#ifdef LOVE_ANDROID
+	if (!love::android::createStorageDirectories())
+		SDL_Log("Error creating storage directories!");
+
+	new_search_path = love::android::getSelectedGameFile();
+
+	// try mounting first, if that fails, load to memory and mount
+	if (!PHYSFS_mount(new_search_path.c_str(), nullptr, 1))
+	{
+		// PHYSFS cannot yet mount a zip file inside an .apk
+		SDL_Log("Mounting %s did not work. Loading to memory.",
+				new_search_path.c_str());
+		char* game_archive_ptr = NULL;
+		size_t game_archive_size = 0;
+		if (!love::android::loadGameArchiveToMemory(
+					new_search_path.c_str(), &game_archive_ptr,
+					&game_archive_size))
+		{
+			SDL_Log("Failure memory loading archive %s", new_search_path.c_str());
+			return false;
+		}
+		if (!PHYSFS_mountMemory(
+			    game_archive_ptr, game_archive_size,
+			    love::android::freeGameArchiveMemory, "archive.zip", "/", 0))
+		{
+			SDL_Log("Failure mounting in-memory archive.");
+			love::android::freeGameArchiveMemory(game_archive_ptr);
+			return false;
+		}
+	}
+#else
 	// Add the directory.
-	if (!PHYSFS_addToSearchPath(source, 1))
+	if (!PHYSFS_mount(new_search_path.c_str(), nullptr, 1))
 		return false;
+#endif
 
 	// Save the game source.
-	game_source = std::string(source);
+	game_source = new_search_path;
 
 	return true;
 }
@@ -164,23 +269,43 @@ const char *Filesystem::getSource() const
 
 bool Filesystem::setupWriteDirectory()
 {
-	if (!initialized)
+	if (!PHYSFS_isInit())
 		return false;
 
 	// These must all be set.
 	if (save_identity.empty() || save_path_full.empty() || save_path_relative.empty())
 		return false;
 
-	// Set the appdata folder as writable directory.
+	// We need to make sure the write directory is created. To do that, we also
+	// need to make sure all its parent directories are also created.
+	std::string temp_writedir = getDriveRoot(save_path_full);
+	std::string temp_createdir = skipDriveRoot(save_path_full);
+
+	// On some sandboxed platforms, physfs will break when its write directory
+	// is the root of the drive and it tries to create a folder (even if the
+	// folder's path is in a writable location.) If the user's home folder is
+	// in the save path, we'll try starting from there instead.
+	if (save_path_full.find(getUserDirectory()) == 0)
+	{
+		temp_writedir = getUserDirectory();
+		temp_createdir = save_path_full.substr(getUserDirectory().length());
+
+		// Strip leading '/' characters from the path we want to create.
+		size_t startpos = temp_createdir.find_first_not_of('/');
+		if (startpos != std::string::npos)
+			temp_createdir = temp_createdir.substr(startpos);
+	}
+
+	// Set either '/' or the user's home as a writable directory.
 	// (We must create the save folder before mounting it).
-	if (!PHYSFS_setWriteDir(getDriveRoot(save_path_full).c_str()))
+	if (!PHYSFS_setWriteDir(temp_writedir.c_str()))
 		return false;
 
-	// Create the save folder. (We're now "at" %APPDATA%).
-	if (!createDirectory(skipDriveRoot(save_path_full).c_str()))
+	// Create the save folder. (We're now "at" either '/' or the user's home).
+	if (!createDirectory(temp_createdir.c_str()))
 	{
 		// Clear the write directory in case of error.
-		PHYSFS_setWriteDir(0);
+		PHYSFS_setWriteDir(nullptr);
 		return false;
 	}
 
@@ -189,9 +314,9 @@ bool Filesystem::setupWriteDirectory()
 		return false;
 
 	// Add the directory. (Will not be readded if already present).
-	if (!PHYSFS_addToSearchPath(save_path_full.c_str(), 0))
+	if (!PHYSFS_mount(save_path_full.c_str(), nullptr, 0))
 	{
-		PHYSFS_setWriteDir(0); // Clear the write directory in case of error.
+		PHYSFS_setWriteDir(nullptr); // Clear the write directory in case of error.
 		return false;
 	}
 
@@ -200,13 +325,18 @@ bool Filesystem::setupWriteDirectory()
 
 bool Filesystem::mount(const char *archive, const char *mountpoint, bool appendToPath)
 {
-	if (!initialized || !archive)
+	if (!PHYSFS_isInit() || !archive)
 		return false;
 
 	std::string realPath;
 	std::string sourceBase = getSourceBaseDirectory();
 
-	if (isFused() && sourceBase.compare(archive) == 0)
+	// Check whether the given archive path is in the list of allowed full paths.
+	auto it = std::find(allowedMountPaths.begin(), allowedMountPaths.end(), archive);
+
+	if (it != allowedMountPaths.end())
+		realPath = *it;
+	else if (isFused() && sourceBase.compare(archive) == 0)
 	{
 		// Special case: if the game is fused and the archive is the source's
 		// base directory, mount it even though it's outside of the save dir.
@@ -236,18 +366,45 @@ bool Filesystem::mount(const char *archive, const char *mountpoint, bool appendT
 	if (realPath.length() == 0)
 		return false;
 
-	return PHYSFS_mount(realPath.c_str(), mountpoint, appendToPath);
+	return PHYSFS_mount(realPath.c_str(), mountpoint, appendToPath) != 0;
+}
+
+bool Filesystem::mount(Data *data, const char *archivename, const char *mountpoint, bool appendToPath)
+{
+	if (!PHYSFS_isInit())
+		return false;
+
+	if (PHYSFS_mountMemory(data->getData(), data->getSize(), nullptr, archivename, mountpoint, appendToPath) != 0)
+	{
+		mountedData[archivename] = data;
+		return true;
+	}
+
+	return false;
 }
 
 bool Filesystem::unmount(const char *archive)
 {
-	if (!initialized || !archive)
+	if (!PHYSFS_isInit() || !archive)
 		return false;
 
+	auto datait = mountedData.find(archive);
+
+	if (datait != mountedData.end() && PHYSFS_unmount(archive) != 0)
+	{
+		mountedData.erase(datait);
+		return true;
+	}
+
 	std::string realPath;
 	std::string sourceBase = getSourceBaseDirectory();
 
-	if (isFused() && sourceBase.compare(archive) == 0)
+	// Check whether the given archive path is in the list of allowed full paths.
+	auto it = std::find(allowedMountPaths.begin(), allowedMountPaths.end(), archive);
+
+	if (it != allowedMountPaths.end())
+		realPath = *it;
+	else if (isFused() && sourceBase.compare(archive) == 0)
 	{
 		// Special case: if the game is fused and the archive is the source's
 		// base directory, unmount it even though it's outside of the save dir.
@@ -272,38 +429,28 @@ bool Filesystem::unmount(const char *archive)
 	if (!mountPoint)
 		return false;
 
-	return PHYSFS_removeFromSearchPath(realPath.c_str());
+	return PHYSFS_unmount(realPath.c_str()) != 0;
 }
 
-File *Filesystem::newFile(const char *filename) const
+bool Filesystem::unmount(Data *data)
+{
+	for (const auto &datapair : mountedData)
+	{
+		if (datapair.second.get() == data)
+		{
+			std::string archive = datapair.first;
+			return unmount(archive.c_str());
+		}
+	}
+
+	return false;
+}
+
+love::filesystem::File *Filesystem::newFile(const char *filename) const
 {
 	return new File(filename);
 }
 
-FileData *Filesystem::newFileData(void *data, unsigned int size, const char *filename) const
-{
-	FileData *fd = new FileData(size, std::string(filename));
-
-	// Copy the data into FileData.
-	memcpy(fd->getData(), data, size);
-
-	return fd;
-}
-
-FileData *Filesystem::newFileData(const char *b64, const char *filename) const
-{
-	int size = strlen(b64);
-	int outsize = 0;
-	char *dst = b64_decode(b64, size, outsize);
-	FileData *fd = new FileData(outsize, std::string(filename));
-
-	// Copy the data into FileData.
-	memcpy(fd->getData(), dst, outsize);
-	delete [] dst;
-
-	return fd;
-}
-
 const char *Filesystem::getWorkingDirectory()
 {
 	if (cwd.empty())
@@ -327,42 +474,45 @@ const char *Filesystem::getWorkingDirectory()
 	return cwd.c_str();
 }
 
-const char *Filesystem::getUserDirectory()
+std::string Filesystem::getUserDirectory()
 {
-	return PHYSFS_getUserDir();
+#ifdef LOVE_IOS
+	// PHYSFS_getUserDir doesn't give exactly the path we want on iOS.
+	static std::string userDir = normalize(love::ios::getHomeDirectory());
+#else
+	static std::string userDir = normalize(PHYSFS_getUserDir());
+#endif
+
+	return userDir;
 }
 
-const char *Filesystem::getAppdataDirectory()
+std::string Filesystem::getAppdataDirectory()
 {
-#ifdef LOVE_WINDOWS
 	if (appdata.empty())
 	{
+#ifdef LOVE_WINDOWS_UWP
+		appdata = getUserDirectory();
+#elif defined(LOVE_WINDOWS)
 		wchar_t *w_appdata = _wgetenv(L"APPDATA");
 		appdata = to_utf8(w_appdata);
 		replace_char(appdata, '\\', '/');
-	}
-	return appdata.c_str();
 #elif defined(LOVE_MACOSX)
-	if (appdata.empty())
-	{
 		std::string udir = getUserDirectory();
 		udir.append("/Library/Application Support");
-		appdata = udir;
-	}
-	return appdata.c_str();
+		appdata = normalize(udir);
+#elif defined(LOVE_IOS)
+		appdata = normalize(love::ios::getAppdataDirectory());
 #elif defined(LOVE_LINUX)
-	if (appdata.empty())
-	{
 		char *xdgdatahome = getenv("XDG_DATA_HOME");
 		if (!xdgdatahome)
-			appdata = std::string(getUserDirectory()) + "/.local/share/";
+			appdata = normalize(std::string(getUserDirectory()) + "/.local/share/");
 		else
 			appdata = xdgdatahome;
-	}
-	return appdata.c_str();
 #else
-	return getUserDirectory();
+		appdata = getUserDirectory();
 #endif
+	}
+	return appdata;
 }
 
 
@@ -399,54 +549,74 @@ std::string Filesystem::getSourceBaseDirectory() const
 
 std::string Filesystem::getRealDirectory(const char *filename) const
 {
+	if (!PHYSFS_isInit())
+		throw love::Exception("PhysFS is not initialized.");
+
 	const char *dir = PHYSFS_getRealDir(filename);
 
 	if (dir == nullptr)
-		throw love::Exception("File does not exist.");
+		throw love::Exception("File does not exist on disk.");
 
 	return std::string(dir);
 }
 
-bool Filesystem::exists(const char *file) const
+bool Filesystem::getInfo(const char *filepath, Info &info) const
 {
-	return PHYSFS_exists(file);
-}
+	if (!PHYSFS_isInit())
+		return false;
 
-bool Filesystem::isDirectory(const char *file) const
-{
-	return PHYSFS_isDirectory(file);
-}
+	PHYSFS_Stat stat = {};
+	if (!PHYSFS_stat(filepath, &stat))
+		return false;
 
-bool Filesystem::isFile(const char *file) const
-{
-	return exists(file) && !isDirectory(file);
+	info.size = (int64) stat.filesize;
+	info.modtime = (int64) stat.modtime;
+
+	if (stat.filetype == PHYSFS_FILETYPE_REGULAR)
+		info.type = FILETYPE_FILE;
+	else if (stat.filetype == PHYSFS_FILETYPE_DIRECTORY)
+		info.type = FILETYPE_DIRECTORY;
+	else if (stat.filetype == PHYSFS_FILETYPE_SYMLINK)
+		info.type = FILETYPE_SYMLINK;
+	else
+		info.type = FILETYPE_OTHER;
+
+	return true;
 }
 
 bool Filesystem::createDirectory(const char *dir)
 {
+	if (!PHYSFS_isInit())
+		return false;
+
 	if (PHYSFS_getWriteDir() == 0 && !setupWriteDirectory())
 		return false;
 
 	if (!PHYSFS_mkdir(dir))
 		return false;
+
 	return true;
 }
 
 bool Filesystem::remove(const char *file)
 {
+	if (!PHYSFS_isInit())
+		return false;
+
 	if (PHYSFS_getWriteDir() == 0 && !setupWriteDirectory())
 		return false;
 
 	if (!PHYSFS_delete(file))
 		return false;
+
 	return true;
 }
 
-Data *Filesystem::read(const char *filename, int64 size) const
+FileData *Filesystem::read(const char *filename, int64 size) const
 {
 	File file(filename);
 
-	file.open(File::READ);
+	file.open(File::MODE_READ);
 
 	// close() is called in the File destructor.
 	return file.read(size);
@@ -456,7 +626,7 @@ void Filesystem::write(const char *filename, const void *data, int64 size) const
 {
 	File file(filename);
 
-	file.open(File::WRITE);
+	file.open(File::MODE_WRITE);
 
 	// close() is called in the File destructor.
 	if (!file.write(data, size))
@@ -467,162 +637,59 @@ void Filesystem::append(const char *filename, const void *data, int64 size) cons
 {
 	File file(filename);
 
-	file.open(File::APPEND);
+	file.open(File::MODE_APPEND);
 
 	// close() is called in the File destructor.
 	if (!file.write(data, size))
 		throw love::Exception("Data could not be written.");
 }
 
-int Filesystem::getDirectoryItems(lua_State *L)
+void Filesystem::getDirectoryItems(const char *dir, std::vector<std::string> &items)
 {
-	const char *dir = luaL_checkstring(L, 1);
-	bool hascallback = !lua_isnoneornil(L, 2);
-
-	if (hascallback)
-		luaL_checktype(L, 2, LUA_TFUNCTION);
+	if (!PHYSFS_isInit())
+		return;
 
 	char **rc = PHYSFS_enumerateFiles(dir);
-	int index = 1;
 
-	lua_newtable(L);
+	if (rc == nullptr)
+		return;
 
 	for (char **i = rc; *i != 0; i++)
-	{
-		if (hascallback)
-		{
-			lua_pushvalue(L, 2);
-			lua_pushstring(L, *i);
-			lua_call(L, 1, 0);
-		}
-
-		lua_pushstring(L, *i);
-		lua_rawseti(L, -2, index);
-		index++;
-	}
+		items.push_back(*i);
 
 	PHYSFS_freeList(rc);
-
-	return 1;
 }
 
-int Filesystem::lines_i(lua_State *L)
+void Filesystem::setSymlinksEnabled(bool enable)
 {
-	const int bufsize = 1024;
-	char buf[bufsize];
-	int linesize = 0;
-	bool newline = false;
+	if (!PHYSFS_isInit())
+		return;
 
-	File *file = luax_checktype<File>(L, lua_upvalueindex(1), "File", FILESYSTEM_FILE_T);
-
-	// Only accept read mode at this point.
-	if (file->getMode() != File::READ)
-		return luaL_error(L, "File needs to stay in read mode.");
-
-	int64 pos = file->tell();
-	int64 userpos = -1;
-
-	if (lua_isnoneornil(L, lua_upvalueindex(2)) == 0)
-	{
-		// User may have changed the file position.
-		userpos = pos;
-		pos = (int64) lua_tonumber(L, lua_upvalueindex(2));
-		if (userpos != pos)
-			file->seek(pos);
-	}
-
-	while (!newline && !file->eof())
-	{
-		// This 64-bit to 32-bit integer cast should be safe as it never exceeds bufsize.
-		int read = (int) file->read(buf, bufsize);
-		if (read < 0)
-			return luaL_error(L, "Could not read from file.");
-
-		linesize += read;
-
-		for (int i = 0; i < read; i++)
-		{
-			if (buf[i] == '\n')
-			{
-				linesize -= read - i;
-				newline = true;
-				break;
-			}
-		}
-	}
-
-	if (newline || (file->eof() && linesize > 0))
-	{
-		if (linesize < bufsize)
-		{
-			// We have the line in the buffer on the stack. No 'new' and 'read' needed.
-			lua_pushlstring(L, buf, linesize > 0 && buf[linesize - 1] == '\r' ? linesize - 1 : linesize);
-			if (userpos < 0)
-				file->seek(pos + linesize + 1);
-		}
-		else
-		{
-			char *str = 0;
-			try
-			{
-				str = new char[linesize + 1];
-			}
-			catch(std::bad_alloc &)
-			{
-				// Can't lua_error (longjmp) in exception handlers.
-			}
-
-			if (!str)
-				return luaL_error(L, "Out of memory.");
-
-			file->seek(pos);
-
-			// Read the \n anyway and save us a call to seek.
-			if (file->read(str, linesize + 1) == -1)
-			{
-				delete [] str;
-				return luaL_error(L, "Could not read from file.");
-			}
-
-			lua_pushlstring(L, str, str[linesize - 1] == '\r' ? linesize - 1 : linesize);
-			delete [] str;
-		}
-
-		if (userpos >= 0)
-		{
-			// Save new position in upvalue.
-			lua_pushnumber(L, (lua_Number)(pos + linesize + 1));
-			lua_replace(L, lua_upvalueindex(2));
-			file->seek(userpos);
-		}
-
-		return 1;
-	}
-
-	// EOF reached.
-	if (userpos >= 0 && luax_toboolean(L, lua_upvalueindex(3)))
-		file->seek(userpos);
-	else
-		file->close();
-
-	return 0;
+	PHYSFS_permitSymbolicLinks(enable ? 1 : 0);
 }
 
-int64 Filesystem::getLastModified(const char *filename) const
+bool Filesystem::areSymlinksEnabled() const
 {
-	PHYSFS_sint64 time = PHYSFS_getLastModTime(filename);
+	if (!PHYSFS_isInit())
+		return false;
 
-	if (time == -1)
-		throw love::Exception("Could not determine file modification date.");
-
-	return time;
+	return PHYSFS_symbolicLinksPermitted() != 0;
 }
 
-int64 Filesystem::getSize(const char *filename) const
+std::vector<std::string> &Filesystem::getRequirePath()
 {
-	File file(filename);
-	int64 size = file.getSize();
-	return size;
+	return requirePath;
+}
+
+std::vector<std::string> &Filesystem::getCRequirePath()
+{
+	return cRequirePath;
+}
+
+void Filesystem::allowMountingForPath(const std::string &path)
+{
+	if (std::find(allowedMountPaths.begin(), allowedMountPaths.end(), path) == allowedMountPaths.end())
+		allowedMountPaths.push_back(path);
 }
 
 } // physfs
