@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2015 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -33,24 +33,31 @@ namespace love
 namespace font
 {
 
-GlyphData::GlyphData(uint32 glyph, GlyphMetrics glyphMetrics, GlyphData::Format f)
+love::Type GlyphData::type("GlyphData", &Data::type);
+
+GlyphData::GlyphData(uint32 glyph, GlyphMetrics glyphMetrics, PixelFormat f)
 	: glyph(glyph)
 	, metrics(glyphMetrics)
 	, data(nullptr)
 	, format(f)
+{
+	if (f != PIXELFORMAT_LA8 && f != PIXELFORMAT_RGBA8)
+		throw love::Exception("Invalid GlyphData pixel format.");
+
+	if (metrics.width > 0 && metrics.height > 0)
+		data = new uint8[metrics.width * metrics.height * getPixelSize()];
+}
+
+GlyphData::GlyphData(const GlyphData &c)
+	: glyph(c.glyph)
+	, metrics(c.metrics)
+	, data(nullptr)
+	, format(c.format)
 {
 	if (metrics.width > 0 && metrics.height > 0)
 	{
-		switch (f)
-		{
-		case GlyphData::FORMAT_LUMINANCE_ALPHA:
-			data = new unsigned char[metrics.width * metrics.height * 2];
-			break;
-		case GlyphData::FORMAT_RGBA:
-		default:
-			data = new unsigned char[metrics.width * metrics.height * 4];
-			break;
-		}
+		data = new uint8[metrics.width * metrics.height * getPixelSize()];
+		memcpy(data, c.data, c.getSize());
 	}
 }
 
@@ -59,24 +66,30 @@ GlyphData::~GlyphData()
 	delete[] data;
 }
 
+GlyphData *GlyphData::clone() const
+{
+	return new GlyphData(*this);
+}
+
 void *GlyphData::getData() const
 {
-	return (void *) data;
+	return data;
+}
+
+size_t GlyphData::getPixelSize() const
+{
+	return getPixelFormatSize(format);
+}
+
+void *GlyphData::getData(int x, int y) const
+{
+	size_t offset = (y * getWidth() + x) * getPixelSize();
+	return data + offset;
 }
 
 size_t GlyphData::getSize() const
 {
-	switch (format)
-	{
-	case GlyphData::FORMAT_LUMINANCE_ALPHA:
-		return size_t(getWidth() * getHeight() * 2);
-		break;
-	case GlyphData::FORMAT_RGBA:
-	default:
-		return size_t(getWidth() * getHeight() * 4);
-		break;
-	}
-
+	return size_t(getWidth() * getHeight()) * getPixelSize();
 }
 
 int GlyphData::getHeight() const
@@ -133,46 +146,28 @@ int GlyphData::getBearingY() const
 
 int GlyphData::getMinX() const
 {
-	return this->getBearingX();
+	return getBearingX();
 }
 
 int GlyphData::getMinY() const
 {
-	return this->getHeight() - this->getBearingY();
+	return getHeight() - getBearingY();
 }
 
 int GlyphData::getMaxX() const
 {
-	return this->getBearingX() + this->getWidth();
+	return getBearingX() + getWidth();
 }
 
 int GlyphData::getMaxY() const
 {
-	return this->getBearingY();
+	return getBearingY();
 }
 
-GlyphData::Format GlyphData::getFormat() const
+PixelFormat GlyphData::getFormat() const
 {
 	return format;
 }
 
-bool GlyphData::getConstant(const char *in, GlyphData::Format &out)
-{
-	return formats.find(in, out);
-}
-
-bool GlyphData::getConstant(GlyphData::Format in, const char *&out)
-{
-	return formats.find(in, out);
-}
-
-StringMap<GlyphData::Format, GlyphData::FORMAT_MAX_ENUM>::Entry GlyphData::formatEntries[] =
-{
-	{"luminance alpha", GlyphData::FORMAT_LUMINANCE_ALPHA},
-	{"rgba", GlyphData::FORMAT_RGBA},
-};
-
-StringMap<GlyphData::Format, GlyphData::FORMAT_MAX_ENUM> GlyphData::formats(GlyphData::formatEntries, sizeof(GlyphData::formatEntries));
-
 } // font
 } // love
