@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -27,6 +27,7 @@
 #include "common/Exception.h"
 #include "common/StringMap.h"
 #include "common/int.h"
+#include "common/pixelformat.h"
 
 // stdlib
 #include <string>
@@ -55,20 +56,27 @@ class GlyphData : public Data
 {
 public:
 
-	enum Format
-	{
-		FORMAT_LUMINANCE_ALPHA,
-		FORMAT_RGBA,
-		FORMAT_MAX_ENUM
-	};
+	static love::Type type;
 
-	GlyphData(uint32 glyph, GlyphMetrics glyphMetrics, Format f);
+	GlyphData(uint32 glyph, GlyphMetrics glyphMetrics, PixelFormat f);
+	GlyphData(const GlyphData &c);
 	virtual ~GlyphData();
 
 	// Implements Data.
+	GlyphData *clone() const;
 	void *getData() const;
 	size_t getSize() const;
 
+	/**
+	 * Gets the data starting at a specific pixel in the glyph.
+	 **/
+	void *getData(int x, int y) const;
+
+	/**
+	 * Gets the size in bytes of each pixel in the glyph.
+	 **/
+	size_t getPixelSize() const;
+
 	/**
 	 * Gets the height of the glyph.
 	 **/
@@ -127,10 +135,7 @@ public:
 	/**
 	 * Gets the format of the glyph data.
 	 **/
-	Format getFormat() const;
-
-	static bool getConstant(const char *in, Format &out);
-	static bool getConstant(Format in, const char *&out);
+	PixelFormat getFormat() const;
 
 private:
 
@@ -141,13 +146,10 @@ private:
 	GlyphMetrics metrics;
 
 	// Glyph texture data.
-	unsigned char *data;
+	uint8 *data;
 
 	// The format the data's in.
-	Format format;
-
-	static StringMap<Format, FORMAT_MAX_ENUM>::Entry formatEntries[];
-	static StringMap<Format, FORMAT_MAX_ENUM> formats;
+	PixelFormat format;
 
 }; // GlyphData
 
