@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -25,11 +25,7 @@
 #include "font/Font.h"
 
 // FreeType2
-#ifdef LOVE_MACOSX_USE_FRAMEWORKS
-#include <freetype/ft2build.h>
-#else
 #include <ft2build.h>
-#endif
 #include FT_FREETYPE_H
 #include FT_GLYPH_H
 
@@ -45,26 +41,21 @@ class Font : public love::font::Font
 public:
 
 	Font();
-
-	/**
-	 * Destructor.
-	 **/
 	virtual ~Font();
 
 	// Implements Font
-	Rasterizer *newRasterizer(Data *data, int size);
-	Rasterizer *newRasterizer(love::image::ImageData *data, const std::string &text);
-	Rasterizer *newRasterizer(love::image::ImageData *data, uint32 *glyphs, int numglyphs);
-	GlyphData *newGlyphData(Rasterizer *r, const std::string &glyph);
-	GlyphData *newGlyphData(Rasterizer *r, uint32 glyph);
+	Rasterizer *newRasterizer(love::filesystem::FileData *data) override;
+	Rasterizer *newTrueTypeRasterizer(love::Data *data, int size, TrueTypeRasterizer::Hinting hinting) override;
+	Rasterizer *newTrueTypeRasterizer(love::Data *data, int size, float dpiscale, TrueTypeRasterizer::Hinting hinting) override;
 
 	// Implement Module
-	const char *getName() const;
+	const char *getName() const override;
 
 private:
 
 	// FreeType library
 	FT_Library library;
+
 }; // Font
 
 } // freetype
