@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2016 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -29,11 +29,13 @@ namespace love
 namespace graphics
 {
 
+love::Type Quad::type("Quad", &Object::type);
+
 Quad::Quad(const Quad::Viewport &v, double sw, double sh)
 	: sw(sw)
 	, sh(sh)
 {
-	memset(vertices, 255, sizeof(Vertex) * 4);
+	arrayLayer = 0;
 	refresh(v, sw, sh);
 }
 
@@ -43,30 +45,23 @@ Quad::~Quad()
 
 void Quad::refresh(const Quad::Viewport &v, double sw, double sh)
 {
-	viewport = v;
+	this->viewport = v;
+	this->sw = sw;
+	this->sh = sh;
 
 	// Vertices are ordered for use with triangle strips:
-	// 0----2
-	// |  / |
-	// | /  |
-	// 1----3
-	vertices[0].x = 0.0f;
-	vertices[0].y = 0.0f;
-	vertices[1].x = 0.0f;
-	vertices[1].y = (float) v.h;
-	vertices[2].x = (float) v.w;
-	vertices[2].y = 0.0f;
-	vertices[3].x = (float) v.w;
-	vertices[3].y = (float) v.h;
+	// 0---2
+	// | / |
+	// 1---3
+	vertexPositions[0] = Vector2(0.0f, 0.0f);
+	vertexPositions[1] = Vector2(0.0f, (float) v.h);
+	vertexPositions[2] = Vector2((float) v.w, 0.0f);
+	vertexPositions[3] = Vector2((float) v.w, (float) v.h);
 
-	vertices[0].s = (float) (v.x/sw);
-	vertices[0].t = (float) (v.y/sh);
-	vertices[1].s = (float) (v.x/sw);
-	vertices[1].t = (float) ((v.y+v.h)/sh);
-	vertices[2].s = (float) ((v.x+v.w)/sw);
-	vertices[2].t = (float) (v.y/sh);
-	vertices[3].s = (float) ((v.x+v.w)/sw);
-	vertices[3].t = (float) ((v.y+v.h)/sh);
+	vertexTexCoords[0] = Vector2((float) (v.x / sw), (float) (v.y / sh));
+	vertexTexCoords[1] = Vector2((float) (v.x / sw), (float) ((v.y + v.h) / sh));
+	vertexTexCoords[2] = Vector2((float) ((v.x + v.w) / sw), (float) (v.y / sh));
+	vertexTexCoords[3] = Vector2((float) ((v.x + v.w) / sw), (float) ((v.y + v.h) / sh));
 }
 
 void Quad::setViewport(const Quad::Viewport &v)
@@ -89,9 +84,14 @@ double Quad::getTextureHeight() const
 	return sh;
 }
 
-const Vertex *Quad::getVertices() const
+void Quad::setLayer(int layer)
 {
-	return vertices;
+	arrayLayer = layer;
+}
+
+int Quad::getLayer() const
+{
+	return arrayLayer;
 }
 
 } // graphics
