@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -24,6 +24,7 @@
 // LOVE
 #include "common/Object.h"
 #include "common/math.h"
+#include "common/Vector.h"
 
 namespace love
 {
@@ -34,30 +35,40 @@ class Quad : public Object
 {
 public:
 
+	static love::Type type;
+
 	struct Viewport
 	{
-		float x, y;
-		float w, h;
+		double x, y;
+		double w, h;
 	};
 
-	static const size_t NUM_VERTICES = 4;
-
-	Quad(const Viewport &v, float sw, float sh);
+	Quad(const Viewport &v, double sw, double sh);
 	virtual ~Quad();
 
-	void refresh(const Viewport &v, float sw, float sh);
+	void refresh(const Viewport &v, double sw, double sh);
 	void setViewport(const Viewport &v);
 	Viewport getViewport() const;
 
-	const Vertex *getVertices() const;
+	double getTextureWidth() const;
+	double getTextureHeight() const;
+
+	const Vector2 *getVertexPositions() const { return vertexPositions; }
+	const Vector2 *getVertexTexCoords() const { return vertexTexCoords; }
+
+	void setLayer(int layer);
+	int getLayer() const;
 
 private:
 
-	Vertex vertices[NUM_VERTICES];
+	Vector2 vertexPositions[4];
+	Vector2 vertexTexCoords[4];
+
+	int arrayLayer;
 
 	Viewport viewport;
-	float sw;
-	float sh;
+	double sw;
+	double sh;
 
 }; // Quad
 
