@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -43,26 +43,17 @@ Volatile::~Volatile()
 bool Volatile::loadAll()
 {
 	bool success = true;
-	std::list<Volatile *>::iterator i = all.begin();
 
-	while (i != all.end())
-	{
-		success = success && (*i)->loadVolatile();
-		i++;
-	}
+	for (Volatile *v : all)
+		success = success && v->loadVolatile();
 
 	return success;
 }
 
 void Volatile::unloadAll()
 {
-	std::list<Volatile *>::iterator i = all.begin();
-
-	while (i != all.end())
-	{
-		(*i)->unloadVolatile();
-		i++;
-	}
+	for (Volatile *v : all)
+		v->unloadVolatile();
 }
 
 } // graphics
