@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2016 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -28,6 +28,9 @@
 #include "ImageData.h"
 #include "CompressedImageData.h"
 
+// C++
+#include <list>
+
 namespace love
 {
 namespace image
@@ -44,17 +47,21 @@ class Image : public Module
 {
 public:
 
-	virtual ~Image() {}
+	static love::Type type;
+
+	Image();
+	virtual ~Image();
 
 	// Implements Module.
-	virtual ModuleType getModuleType() const { return M_IMAGE; }
+	ModuleType getModuleType() const override { return M_IMAGE; }
+	const char *getName() const override;
 
 	/**
 	 * Creates new ImageData from FileData.
 	 * @param data The FileData containing the encoded image data.
 	 * @return The new ImageData.
 	 **/
-	virtual ImageData *newImageData(love::filesystem::FileData *data) = 0;
+	ImageData *newImageData(Data *data);
 
 	/**
 	 * Creates empty ImageData with the given size.
@@ -62,7 +69,7 @@ public:
 	 * @param height The height of the ImageData.
 	 * @return The new ImageData.
 	 **/
-	virtual ImageData *newImageData(int width, int height) = 0;
+	ImageData *newImageData(int width, int height, PixelFormat format = PIXELFORMAT_RGBA8);
 
 	/**
 	 * Creates empty ImageData with the given size.
@@ -73,20 +80,32 @@ public:
 	 *        copy it.
 	 * @return The new ImageData.
 	 **/
-	virtual ImageData *newImageData(int width, int height, void *data, bool own = false) = 0;
+	ImageData *newImageData(int width, int height, PixelFormat format, void *data, bool own = false);
 
 	/**
 	 * Creates new CompressedImageData from FileData.
 	 * @param data The FileData containing the compressed image data.
 	 * @return The new CompressedImageData.
 	 **/
-	virtual CompressedImageData *newCompressedData(love::filesystem::FileData *data) = 0;
+	CompressedImageData *newCompressedData(Data *data);
 
 	/**
 	 * Determines whether a FileData is Compressed image data or not.
 	 * @param data The FileData to test.
 	 **/
-	virtual bool isCompressed(love::filesystem::FileData *data) = 0;
+	bool isCompressed(Data *data);
+
+	std::vector<StrongRef<ImageData>> newCubeFaces(ImageData *src);
+	std::vector<StrongRef<ImageData>> newVolumeLayers(ImageData *src);
+
+	const std::list<FormatHandler *> &getFormatHandlers() const;
+
+private:
+
+	ImageData *newPastedImageData(ImageData *src, int sx, int sy, int w, int h);
+
+	// Image format handlers we can use for decoding and encoding ImageData.
+	std::list<FormatHandler *> formatHandlers;
 
 }; // Image
 
