@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2015 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -38,6 +38,8 @@ class Joystick : public Object
 {
 public:
 
+	static love::Type type;
+
 	// Joystick hat values.
 	enum Hat
 	{
@@ -150,6 +152,8 @@ public:
 	virtual float getGamepadAxis(GamepadAxis axis) const = 0;
 	virtual bool isGamepadDown(const std::vector<GamepadButton> &blist) const = 0;
 
+	virtual JoystickInput getGamepadMapping(const GamepadInput &input) const = 0;
+
 	virtual void *getHandle() const = 0;
 
 	virtual std::string getGUID() const = 0;
