@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2015 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -32,23 +32,6 @@ namespace joystick
 {
 
 Joystick *luax_checkjoystick(lua_State *L, int idx);
-int w_Joystick_isConnected(lua_State *L);
-int w_Joystick_getName(lua_State *L);
-int w_Joystick_getID(lua_State *L);
-int w_Joystick_getGUID(lua_State *L);
-int w_Joystick_getAxisCount(lua_State *L);
-int w_Joystick_getButtonCount(lua_State *L);
-int w_Joystick_getHatCount(lua_State *L);
-int w_Joystick_getAxis(lua_State *L);
-int w_Joystick_getAxes(lua_State *L);
-int w_Joystick_getHat(lua_State *L);
-int w_Joystick_isDown(lua_State *L);
-int w_Joystick_isGamepad(lua_State *L);
-int w_Joystick_getGamepadAxis(lua_State *L);
-int w_Joystick_isGamepadDown(lua_State *L);
-int w_Joystick_isVibrationSupported(lua_State *L);
-int w_Joystick_setVibration(lua_State *L);
-int w_Joystick_getVibration(lua_State *L);
 extern "C" int luaopen_joystick(lua_State *L);
 
 } // joystick
