@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2015 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -35,10 +35,12 @@ class BezierCurve : public Object
 {
 public:
 
+	static love::Type type;
+
 	/**
 	 * @param controlPoints Control polygon of the curve.
 	 **/
-	BezierCurve(const std::vector<Vector> &controlPoints);
+	BezierCurve(const std::vector<Vector2> &controlPoints);
 
 	/**
 	 * @returns Degree of the curve
@@ -56,14 +58,14 @@ public:
 	/**
 	 * @returns i'th control point.
 	 **/
-	const Vector &getControlPoint(int i) const;
+	const Vector2 &getControlPoint(int i) const;
 
 	/**
 	 * Sets the i'th control point.
 	 * @param i Control point to change.
 	 * @param point New control point.
 	 **/
-	void setControlPoint(int i, const Vector &point);
+	void setControlPoint(int i, const Vector2 &point);
 
 	/**
 	 * Insert a new control point before the i'th control point.
@@ -71,7 +73,13 @@ public:
 	 * @param point Control point to insert.
 	 * @param pos Position to insert.
 	 **/
-	void insertControlPoint(const Vector &point, int pos = -1);
+	void insertControlPoint(const Vector2 &point, int pos = -1);
+
+	/**
+	 * Remove the i'th control point from the curve.
+	 * @param i Control point to remove
+	 **/
+	void removeControlPoint(int i);
 
 	/**
 	 * @returns Number of control points.
@@ -85,37 +93,55 @@ public:
 	 * Move the curve.
 	 * @param t Translation vector.
 	 */
-	void translate(const Vector &t);
+	void translate(const Vector2 &t);
 
 	/**
 	 * Rotate the curve.
 	 * @param phi Rotation angle (radians).
 	 * @param center Rotation center.
 	 */
-	void rotate(double phi, const Vector &center);
+	void rotate(double phi, const Vector2 &center);
 
 	/**
 	 * Scale the curve.
 	 * @param phi Scale factor.
 	 * @param center Scale center.
 	 */
-	void scale(double phi, const Vector &center);
+	void scale(double phi, const Vector2 &center);
 
 	/**
 	 * Evaluates the curve at time t.
 	 * @param t Curve parameter, must satisfy 0 <= t <= 1.
 	 **/
-	Vector evaluate(double t) const;
+	Vector2 evaluate(double t) const;
+
+	/**
+	 * Get curve segment starting at t1 and ending at t2.
+	 * The new curve will be parametrized from 0 <= t <= 1.
+	 * @param t1 Start of the segment.
+	 * @param t2 End of the segment.
+	 * @returns Bezier curve covering the segment.
+	 */
+	BezierCurve* getSegment(double t1, double t2) const;
 
 	/**
 	 * Renders the curve by subdivision.
 	 * @param accuracy The 'fineness' of the curve.
 	 * @returns A polygon chain that approximates the bezier curve.
 	 **/
-	std::vector<Vector> render(size_t accuracy = 4) const;
+	std::vector<Vector2> render(int accuracy = 4) const;
+
+	/**
+	 * Renders a segment of the curve by subdivision.
+	 * @param start The starting point (between 0 and 1) on the curve.
+	 * @param end The ending point on the curve.
+	 * @param accuracy The 'fineness' of the curve.
+	 * @returns A polygon chain that approximates the segment along the curve
+	 **/
+	std::vector<Vector2> renderSegment(double start, double end, int accuracy = 4) const;
 
 private:
-	std::vector<Vector> controlPoints;
+	std::vector<Vector2> controlPoints;
 };
 
 }
