@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -24,9 +24,11 @@
 // LOVE
 #include "Cursor.h"
 #include "common/Module.h"
-#include "common/StringMap.h"
 #include "image/ImageData.h"
 
+// C++
+#include <vector>
+
 namespace love
 {
 namespace mouse
@@ -36,21 +38,11 @@ class Mouse : public Module
 {
 public:
 
-	enum Button
-	{
-		BUTTON_INVALID,
-		BUTTON_LEFT,
-		BUTTON_MIDDLE,
-		BUTTON_RIGHT,
-		BUTTON_X1,
-		BUTTON_X2,
-		BUTTON_WHEELUP,
-		BUTTON_WHEELDOWN,
-		BUTTON_MAX_ENUM
-	};
-
 	virtual ~Mouse() {}
 
+	// Implements Module.
+	virtual ModuleType getModuleType() const { return M_MOUSE; }
+
 	virtual Cursor *newCursor(love::image::ImageData *data, int hotx, int hoty) = 0;
 	virtual Cursor *getSystemCursor(Cursor::SystemCursor cursortype) = 0;
 
@@ -59,25 +51,21 @@ public:
 
 	virtual Cursor *getCursor() const = 0;
 
-	virtual int getX() const = 0;
-	virtual int getY() const = 0;
-	virtual void getPosition(int &x, int &y) const = 0;
-	virtual void setX(int x) = 0;
-	virtual void setY(int y) = 0;
-	virtual void setPosition(int x, int y) = 0;
+	virtual bool isCursorSupported() const = 0;
+
+	virtual double getX() const = 0;
+	virtual double getY() const = 0;
+	virtual void getPosition(double &x, double &y) const = 0;
+	virtual void setX(double x) = 0;
+	virtual void setY(double y) = 0;
+	virtual void setPosition(double x, double y) = 0;
 	virtual void setVisible(bool visible) = 0;
-	virtual bool isDown(Button *buttonlist) const = 0;
+	virtual bool isDown(const std::vector<int> &buttons) const = 0;
 	virtual bool isVisible() const = 0;
 	virtual void setGrabbed(bool grab) = 0;
 	virtual bool isGrabbed() const = 0;
-
-	static bool getConstant(const char *in, Button &out);
-	static bool getConstant(Button in, const char  *&out);
-
-private:
-
-	static StringMap<Button, BUTTON_MAX_ENUM>::Entry buttonEntries[];
-	static StringMap<Button, BUTTON_MAX_ENUM> buttons;
+	virtual bool setRelativeMode(bool relative) = 0;
+	virtual bool getRelativeMode() const = 0;
 
 }; // Mouse
 
