@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2015 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -34,25 +34,19 @@ namespace sdl
 
 // SDL reports mouse coordinates in the window coordinate system in OS X, but
 // we want them in pixel coordinates (may be different with high-DPI enabled.)
-static void windowToPixelCoords(int *x, int *y)
+static void windowToDPICoords(double *x, double *y)
 {
-	window::Window *window = Module::getInstance<window::Window>(Module::M_WINDOW);
-
-	if (window && x)
-		*x = (int) window->toPixels(*x);
-	if (window && y)
-		*y = (int) window->toPixels(*y);
+	auto window = Module::getInstance<window::Window>(Module::M_WINDOW);
+	if (window)
+		window->windowToDPICoords(x, y);
 }
 
 // And vice versa for setting mouse coordinates.
-static void pixelToWindowCoords(int *x, int *y)
+static void DPIToWindowCoords(double *x, double *y)
 {
-	window::Window *window = Module::getInstance<window::Window>(Module::M_WINDOW);
-
-	if (window && x)
-		*x = (int) window->fromPixels(*x);
-	if (window && y)
-		*y = (int) window->fromPixels(*y);
+	auto window = Module::getInstance<window::Window>(Module::M_WINDOW);
+	if (window)
+		window->DPIToWindowCoords(x, y);
 }
 
 const char *Mouse::getName() const
@@ -112,44 +106,54 @@ love::mouse::Cursor *Mouse::getCursor() const
 	return curCursor.get();
 }
 
-int Mouse::getX() const
+
+bool Mouse::isCursorSupported() const
+{
+	return SDL_GetDefaultCursor() != nullptr;
+}
+
+double Mouse::getX() const
 {
 	int x;
 	SDL_GetMouseState(&x, nullptr);
-	windowToPixelCoords(&x, nullptr);
 
-	return x;
+	double dx = (double) x;
+	windowToDPICoords(&dx, nullptr);
+
+	return dx;
 }
 
-int Mouse::getY() const
+double Mouse::getY() const
 {
 	int y;
 	SDL_GetMouseState(nullptr, &y);
-	windowToPixelCoords(nullptr, &y);
 
-	return y;
+	double dy = (double) y;
+	windowToDPICoords(nullptr, &dy);
+
+	return dy;
 }
 
-void Mouse::getPosition(int &x, int &y) const
+void Mouse::getPosition(double &x, double &y) const
 {
 	int mx, my;
 	SDL_GetMouseState(&mx, &my);
-	windowToPixelCoords(&mx, &my);
 
-	x = mx;
-	y = my;
+	x = (double) mx;
+	y = (double) my;
+	windowToDPICoords(&x, &y);
 }
 
-void Mouse::setPosition(int x, int y)
+void Mouse::setPosition(double x, double y)
 {
-	love::window::Window *window = love::window::sdl::Window::getSingleton();
+	auto window = Module::getInstance<window::Window>(Module::M_WINDOW);
 
 	SDL_Window *handle = nullptr;
 	if (window)
 		handle = (SDL_Window *) window->getHandle();
 
-	pixelToWindowCoords(&x, &y);
-	SDL_WarpMouseInWindow(handle, x, y);
+	DPIToWindowCoords(&x, &y);
+	SDL_WarpMouseInWindow(handle, (int) x, (int) y);
 
 	// SDL_WarpMouse doesn't directly update SDL's internal mouse state in Linux
 	// and Windows, so we call SDL_PumpEvents now to make sure the next
@@ -157,16 +161,14 @@ void Mouse::setPosition(int x, int y)
 	SDL_PumpEvents();
 }
 
-void Mouse::setX(int x)
+void Mouse::setX(double x)
 {
-	int y = getY();
-	setPosition(x, y);
+	setPosition(x, getY());
 }
 
-void Mouse::setY(int y)
+void Mouse::setY(double y)
 {
-	int x = getX();
-	setPosition(x, y);
+	setPosition(getX(), y);
 }
 
 void Mouse::setVisible(bool visible)
@@ -174,12 +176,27 @@ void Mouse::setVisible(bool visible)
 	SDL_ShowCursor(visible ? SDL_ENABLE : SDL_DISABLE);
 }
 
-bool Mouse::isDown(Button *buttonlist) const
+bool Mouse::isDown(const std::vector<int> &buttons) const
 {
 	Uint32 buttonstate = SDL_GetMouseState(nullptr, nullptr);
 
-	for (Button button = *buttonlist; button != BUTTON_MAX_ENUM; button = *(++buttonlist))
+	for (int button : buttons)
 	{
+		if (button <= 0)
+			continue;
+
+		// We use button index 2 to represent the right mouse button, but SDL
+		// uses 2 to represent the middle mouse button.
+		switch (button)
+		{
+		case 2:
+			button = SDL_BUTTON_RIGHT;
+			break;
+		case 3:
+			button = SDL_BUTTON_MIDDLE;
+			break;
+		}
+
 		if (buttonstate & SDL_BUTTON(button))
 			return true;
 	}
@@ -194,14 +211,14 @@ bool Mouse::isVisible() const
 
 void Mouse::setGrabbed(bool grab)
 {
-	love::window::Window *window = love::window::sdl::Window::getSingleton();
+	auto window = Module::getInstance<window::Window>(Module::M_WINDOW);
 	if (window)
 		window->setMouseGrab(grab);
 }
 
 bool Mouse::isGrabbed() const
 {
-	love::window::Window *window = love::window::sdl::Window::getSingleton();
+	auto window = Module::getInstance<window::Window>(Module::M_WINDOW);
 	if (window)
 		return window->isMouseGrabbed();
 	else
