@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2015 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -24,6 +24,7 @@
 #include "common/config.h"
 
 #include "sdl/Mouse.h"
+#include "filesystem/File.h"
 
 namespace love
 {
@@ -36,16 +37,16 @@ int w_newCursor(lua_State *L)
 {
 	Cursor *cursor = nullptr;
 
-	if (lua_isstring(L, 1) || luax_istype(L, 1, FILESYSTEM_FILE_T) || luax_istype(L, 1, FILESYSTEM_FILE_DATA_T))
+	if (lua_isstring(L, 1) || luax_istype(L, 1, love::filesystem::File::type) || luax_istype(L, 1, love::filesystem::FileData::type))
 		luax_convobj(L, 1, "image", "newImageData");
 
-	love::image::ImageData *data = luax_checktype<love::image::ImageData>(L, 1, "ImageData", IMAGE_IMAGE_DATA_T);
-	int hotx = luaL_optint(L, 2, 0);
-	int hoty = luaL_optint(L, 3, 0);
+	love::image::ImageData *data = luax_checktype<love::image::ImageData>(L, 1);
+	int hotx = (int) luaL_optinteger(L, 2, 0);
+	int hoty = (int) luaL_optinteger(L, 3, 0);
 
 	luax_catchexcept(L, [&](){ cursor = instance()->newCursor(data, hotx, hoty); });
 
-	luax_pushtype(L, "Cursor", MOUSE_CURSOR_T, cursor);
+	luax_pushtype(L, cursor);
 	cursor->release();
 	return 1;
 }
@@ -56,12 +57,12 @@ int w_getSystemCursor(lua_State *L)
 	Cursor::SystemCursor systemCursor;
 
 	if (!Cursor::getConstant(str, systemCursor))
-		return luaL_error(L, "Invalid system cursor type: %s", str);
+		return luax_enumerror(L, "system cursor type", str);
 
 	Cursor *cursor = 0;
 	luax_catchexcept(L, [&](){ cursor = instance()->getSystemCursor(systemCursor); });
 
-	luax_pushtype(L, "Cursor", MOUSE_CURSOR_T, cursor);
+	luax_pushtype(L, cursor);
 	return 1;
 }
 
@@ -84,13 +85,19 @@ int w_getCursor(lua_State *L)
 	Cursor *cursor = instance()->getCursor();
 
 	if (cursor)
-		luax_pushtype(L, "Cursor", MOUSE_CURSOR_T, cursor);
+		luax_pushtype(L, cursor);
 	else
 		lua_pushnil(L);
 
 	return 1;
 }
 
+int w_isCursorSupported(lua_State *L)
+{
+	luax_pushboolean(L, instance()->isCursorSupported());
+	return 1;
+}
+
 int w_getX(lua_State *L)
 {
 	lua_pushnumber(L, instance()->getX());
@@ -105,57 +112,65 @@ int w_getY(lua_State *L)
 
 int w_getPosition(lua_State *L)
 {
-	int x, y;
+	double x, y;
 	instance()->getPosition(x, y);
-	lua_pushinteger(L, x);
-	lua_pushinteger(L, y);
+	lua_pushnumber(L, x);
+	lua_pushnumber(L, y);
 	return 2;
 }
 
 int w_setX(lua_State *L)
 {
-	int x = luaL_checkint(L, 1);
+	double x = luaL_checknumber(L, 1);
 	instance()->setX(x);
 	return 0;
 }
 
 int w_setY(lua_State *L)
 {
-	int y = luaL_checkint(L, 1);
+	double y = luaL_checknumber(L, 1);
 	instance()->setY(y);
 	return 0;
 }
 
 int w_setPosition(lua_State *L)
 {
-	int x = luaL_checkint(L, 1);
-	int y = luaL_checkint(L, 2);
+	double x = luaL_checknumber(L, 1);
+	double y = luaL_checknumber(L, 2);
 	instance()->setPosition(x, y);
 	return 0;
 }
 
 int w_isDown(lua_State *L)
 {
-	Mouse::Button b;
-	unsigned int num = lua_gettop(L);
-	Mouse::Button *buttonlist = new Mouse::Button[num+1];
-	unsigned int counter = 0;
+	bool istable = lua_istable(L, 1);
+	int num = istable ? (int) luax_objlen(L, 1) : lua_gettop(L);
 
-	for (unsigned int i = 0; i < num; i++)
+	std::vector<int> buttons;
+	buttons.reserve(num);
+
+	if (istable)
 	{
-		if (Mouse::getConstant(luaL_checkstring(L, i+1), b))
-			buttonlist[counter++] = b;
+		for (int i = 0; i < num; i++)
+		{
+			lua_rawgeti(L, 1, i + 1);
+			buttons.push_back((int) luaL_checkinteger(L, -1));
+			lua_pop(L, 1);
+		}
+	}
+	else
+	{
+		for (int i = 0; i < num; i++)
+			buttons.push_back((int) luaL_checkinteger(L, i + 1));
 	}
-	buttonlist[counter] = Mouse::BUTTON_MAX_ENUM;
 
-	luax_pushboolean(L, instance()->isDown(buttonlist));
-	delete[] buttonlist;
+	luax_pushboolean(L, instance()->isDown(buttons));
 	return 1;
 }
 
 int w_setVisible(lua_State *L)
 {
-	bool b = luax_toboolean(L, 1);
+	bool b = luax_checkboolean(L, 1);
 	instance()->setVisible(b);
 	return 0;
 }
@@ -168,7 +183,7 @@ int w_isVisible(lua_State *L)
 
 int w_setGrabbed(lua_State *L)
 {
-	bool b = luax_toboolean(L, 1);
+	bool b = luax_checkboolean(L, 1);
 	instance()->setGrabbed(b);
 	return 0;
 }
@@ -181,7 +196,7 @@ int w_isGrabbed(lua_State *L)
 
 int w_setRelativeMode(lua_State *L)
 {
-	bool relative = luax_toboolean(L, 1);
+	bool relative = luax_checkboolean(L, 1);
 	luax_pushboolean(L, instance()->setRelativeMode(relative));
 	return 1;
 }
@@ -199,6 +214,7 @@ static const luaL_Reg functions[] =
 	{ "getSystemCursor", w_getSystemCursor },
 	{ "setCursor", w_setCursor },
 	{ "getCursor", w_getCursor },
+	{ "isCursorSupported", w_isCursorSupported },
 	{ "getX", w_getX },
 	{ "getY", w_getY },
 	{ "setX", w_setX },
@@ -235,7 +251,7 @@ extern "C" int luaopen_love_mouse(lua_State *L)
 	WrappedModule w;
 	w.module = instance;
 	w.name = "mouse";
-	w.flags = MODULE_T;
+	w.type = &Module::type;
 	w.functions = functions;
 	w.types = types;
 
