@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -28,6 +28,9 @@
 #include "World.h"
 #include "Physics.h"
 
+// Needed for luax_pushjoint.
+#include "wrap_Joint.h"
+
 namespace love
 {
 namespace physics
@@ -41,7 +44,6 @@ Body::Body(World *world, b2Vec2 p, Body::Type type)
 {
 	udata = new bodyudata();
 	udata->ref = nullptr;
-	world->retain();
 	b2BodyDef def;
 	def.position = Physics::scaleDown(p);
 	def.userData = (void *) udata;
@@ -57,8 +59,7 @@ Body::Body(b2Body *b)
 	, udata(nullptr)
 {
 	udata = (bodyudata *) b->GetUserData();
-	world = (World *)Memoizer::find(b->GetWorld());
-	world->retain();
+	world = (World *) Memoizer::find(b->GetWorld());
 	// Box2D body holds a reference to the love Body.
 	this->retain();
 	Memoizer::add(body, this);
@@ -66,10 +67,13 @@ Body::Body(b2Body *b)
 
 Body::~Body()
 {
-	if (udata != nullptr)
+	if (!udata)
+		return;
+
+	if (udata->ref)
 		delete udata->ref;
+
 	delete udata;
-	world->release();
 }
 
 float Body::getX()
@@ -418,12 +422,28 @@ bool Body::isFixedRotation() const
 	return body->IsFixedRotation();
 }
 
+bool Body::isTouching(Body *other) const
+{
+	const b2ContactEdge *ce = body->GetContactList();
+	b2Body *otherbody = other->body;
+
+	while (ce != nullptr)
+	{
+		if (ce->other == otherbody && ce->contact != nullptr && ce->contact->IsTouching())
+			return true;
+
+		ce = ce->next;
+	}
+
+	return false;
+}
+
 World *Body::getWorld() const
 {
 	return world;
 }
 
-int Body::getFixtureList(lua_State *L) const
+int Body::getFixtures(lua_State *L) const
 {
 	lua_newtable(L);
 	b2Fixture *f = body->GetFixtureList();
@@ -435,8 +455,7 @@ int Body::getFixtureList(lua_State *L) const
 		Fixture *fixture = (Fixture *)Memoizer::find(f);
 		if (!fixture)
 			throw love::Exception("A fixture has escaped Memoizer!");
-		fixture->retain();
-		luax_pushtype(L, "Fixture", PHYSICS_FIXTURE_T, fixture);
+		luax_pushtype(L, fixture);
 		lua_rawseti(L, -2, i);
 		i++;
 	}
@@ -444,6 +463,55 @@ int Body::getFixtureList(lua_State *L) const
 	return 1;
 }
 
+int Body::getJoints(lua_State *L) const
+{
+	lua_newtable(L);
+	const b2JointEdge *je = body->GetJointList();
+	int i = 1;
+
+	do
+	{
+		if (!je)
+			break;
+
+		Joint *joint = (Joint *) Memoizer::find(je->joint);
+		if (!joint)
+			throw love::Exception("A joint has escaped Memoizer!");
+
+		luax_pushjoint(L, joint);
+		lua_rawseti(L, -2, i);
+		i++;
+	}
+	while ((je = je->next));
+
+	return 1;
+}
+
+int Body::getContacts(lua_State *L) const
+{
+	lua_newtable(L);
+	const b2ContactEdge *ce = body->GetContactList();
+	int i = 1;
+	do
+	{
+		if (!ce)
+			break;
+
+		Contact *contact = (Contact *) Memoizer::find(ce->contact);
+		if (!contact)
+			contact = new Contact(ce->contact);
+		else
+			contact->retain();
+
+		luax_pushtype(L, contact);
+		contact->release();
+		lua_rawseti(L, -2, i);
+		i++;
+	}
+	while ((ce = ce->next));
+	return 1;
+}
+
 b2Vec2 Body::getVector(lua_State *L)
 {
 	love::luax_assert_argc(L, 2, 2);
@@ -473,6 +541,10 @@ void Body::destroy()
 	Memoizer::remove(body);
 	body = NULL;
 
+	// Remove userdata reference to avoid it sticking around after GC
+	if (udata && udata->ref)
+		udata->ref->unref();
+
 	// Box2D body destroyed. Release its reference to the love Body.
 	this->release();
 }
@@ -487,16 +559,10 @@ int Body::setUserData(lua_State *L)
 		body->SetUserData((void *) udata);
 	}
 
-	if (udata->ref != nullptr)
-	{
-		// We set the Reference's lua_State to this one before deleting it, so
-		// it unrefs using the current lua_State's stack. This is necessary
-		// if setUserData is called in a coroutine.
-		udata->ref->setL(L);
-		delete udata->ref;
-	}
+	if(!udata->ref)
+		udata->ref = new Reference();
 
-	udata->ref = new Reference(L);
+	udata->ref->ref(L);
 
 	return 0;
 }
