@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -34,8 +34,10 @@ namespace physics
 namespace box2d
 {
 
-ChainShape::ChainShape(b2ChainShape *c, bool loop, bool own)
-	: Shape(c, own), loop(loop)
+love::Type ChainShape::type("ChainShape", &Shape::type);
+
+ChainShape::ChainShape(b2ChainShape *c, bool own)
+	: Shape(c, own)
 {
 }
 
@@ -45,26 +47,58 @@ ChainShape::~ChainShape()
 
 void ChainShape::setNextVertex(float x, float y)
 {
-	if (loop)
-	{
-		throw love::Exception("Physics error: Can't call setNextVertex on a loop ChainShape");
-		return;
-	}
 	b2Vec2 v(x, y);
 	b2ChainShape *c = (b2ChainShape *)shape;
 	c->SetNextVertex(Physics::scaleDown(v));
 }
 
+void ChainShape::setNextVertex()
+{
+	b2ChainShape *c = (b2ChainShape *)shape;
+	c->m_hasNextVertex = false;
+}
+
 void ChainShape::setPreviousVertex(float x, float y)
 {
-	if (loop)
-	{
-		throw love::Exception("Physics error: Can't call setPreviousVertex on a loop ChainShape");
-		return;
-	}
 	b2Vec2 v(x, y);
 	b2ChainShape *c = (b2ChainShape *)shape;
-	c->SetNextVertex(Physics::scaleDown(v));
+	c->SetPrevVertex(Physics::scaleDown(v));
+}
+
+void ChainShape::setPreviousVertex()
+{
+	b2ChainShape *c = (b2ChainShape *)shape;
+	c->m_hasPrevVertex = false;
+}
+
+bool ChainShape::getNextVertex(float &x, float &y) const
+{
+	b2ChainShape *c = (b2ChainShape *)shape;
+
+	if (c->m_hasNextVertex)
+	{
+		b2Vec2 v = Physics::scaleUp(c->m_nextVertex);
+		x = v.x;
+		y = v.y;
+		return true;
+	}
+
+	return false;
+}
+
+bool ChainShape::getPreviousVertex(float &x, float &y) const
+{
+	b2ChainShape *c = (b2ChainShape *)shape;
+
+	if (c->m_hasPrevVertex)
+	{
+		b2Vec2 v = Physics::scaleUp(c->m_prevVertex);
+		x = v.x;
+		y = v.y;
+		return true;
+	}
+
+	return false;
 }
 
 EdgeShape *ChainShape::getChildEdge(int index) const
@@ -76,7 +110,7 @@ EdgeShape *ChainShape::getChildEdge(int index) const
 	{
 		c->GetChildEdge(e, index);
 	}
-	catch (love::Exception &ex)
+	catch (love::Exception &)
 	{
 		delete e;
 		throw;
@@ -85,12 +119,6 @@ EdgeShape *ChainShape::getChildEdge(int index) const
 	return new EdgeShape(e, true);
 }
 
-int ChainShape::getChildCount() const
-{
-	b2ChainShape *c = (b2ChainShape *)shape;
-	return c->GetChildCount();
-}
-
 int ChainShape::getVertexCount() const
 {
 	b2ChainShape *c = (b2ChainShape *)shape;
