@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -49,6 +49,8 @@ public:
 	friend class World;
 	friend class World::ContactCallback;
 
+	static love::Type type;
+
 	/**
 	 * Creates a new Contact by copying a Box2D contact
 	 * point. It does not store the pointer, but copy the
@@ -148,6 +150,11 @@ public:
 
 	void getChildren(int &childA, int &childB);
 
+	/**
+	 * Gets the Fixtures associated with this Contact.
+	 **/
+	void getFixtures(Fixture *&fixtureA, Fixture *&fixtureB);
+
 private:
 
 	// The Box2D contact.
