@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2015 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -32,6 +32,8 @@ namespace physics
 namespace box2d
 {
 
+love::Type MotorJoint::type("MotorJoint", &Joint::type);
+
 MotorJoint::MotorJoint(Body *body1, Body *body2)
 	: Joint(body1, body2)
 	, joint(NULL)
@@ -42,7 +44,7 @@ MotorJoint::MotorJoint(Body *body1, Body *body2)
 	joint = (b2MotorJoint *) createJoint(&def);
 }
 
-MotorJoint::MotorJoint(Body *body1, Body *body2, float correctionFactor)
+MotorJoint::MotorJoint(Body *body1, Body *body2, float correctionFactor, bool collideConnected)
 	: Joint(body1, body2)
 	, joint(NULL)
 {
@@ -50,6 +52,7 @@ MotorJoint::MotorJoint(Body *body1, Body *body2, float correctionFactor)
 
 	def.Initialize(body1->body, body2->body);
 	def.correctionFactor = correctionFactor;
+	def.collideConnected = collideConnected;
 
 	joint = (b2MotorJoint *) createJoint(&def);
 }
