@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -25,6 +25,8 @@
 #include "World.h"
 #include "Physics.h"
 
+#include <float.h>
+
 namespace love
 {
 namespace physics
@@ -32,10 +34,15 @@ namespace physics
 namespace box2d
 {
 
+love::Type MouseJoint::type("MouseJoint", &Joint::type);
+
 MouseJoint::MouseJoint(Body *body1, float x, float y)
 	: Joint(body1)
 	, joint(NULL)
 {
+	if (body1->getType() == Body::BODY_KINEMATIC)
+		throw love::Exception("Cannot attach a MouseJoint to a kinematic body");
+
 	b2MouseJointDef def;
 
 	def.bodyA = body1->world->getGroundBody();
@@ -73,6 +80,12 @@ float MouseJoint::getMaxForce() const
 
 void MouseJoint::setFrequency(float hz)
 {
+	// This is kind of a crappy check. The frequency is used in an internal
+	// box2d calculation whose result must be > FLT_EPSILON, but other variables
+	// go into that calculation...
+	if (hz <= FLT_EPSILON * 2)
+		throw love::Exception("MouseJoint frequency must be a positive number.");
+
 	joint->SetFrequency(hz);
 }
 
@@ -91,6 +104,16 @@ float MouseJoint::getDampingRatio() const
 	return joint->GetDampingRatio();
 }
 
+Body *MouseJoint::getBodyA() const
+{
+	return Joint::getBodyB();
+}
+
+Body *MouseJoint::getBodyB() const
+{
+	return nullptr;
+}
+
 } // box2d
 } // physics
 } // love
