@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -31,7 +31,7 @@ namespace physics
 namespace box2d
 {
 
-int Physics::meter = Physics::DEFAULT_METER;
+float Physics::meter = Physics::DEFAULT_METER;
 
 const char *Physics::getName() const
 {
@@ -93,6 +93,12 @@ EdgeShape *Physics::newEdgeShape(float x1, float y1, float x2, float y2)
 int Physics::newPolygonShape(lua_State *L)
 {
 	int argc = lua_gettop(L);
+
+	bool istable = lua_istable(L, 1);
+
+	if (istable)
+		argc = (int) luax_objlen(L, 1);
+
 	if (argc % 2 != 0)
 		return luaL_error(L, "Number of vertex components must be a multiple of two.");
 
@@ -103,16 +109,31 @@ int Physics::newPolygonShape(lua_State *L)
 	else if (vcount > b2_maxPolygonVertices)
 		return luaL_error(L, "Expected a maximum of %d vertices, got %d.", b2_maxPolygonVertices, vcount);
 
-	b2PolygonShape *s = new b2PolygonShape();
-
 	b2Vec2 vecs[b2_maxPolygonVertices];
 
-	for (int i = 0; i < vcount; i++)
+	if (istable)
 	{
-		float x = (float)luaL_checknumber(L, 1 + i * 2);
-		float y = (float)luaL_checknumber(L, 2 + i * 2);
-		vecs[i] = Physics::scaleDown(b2Vec2(x, y));
+		for (int i = 0; i < vcount; i++)
+		{
+			lua_rawgeti(L, 1, 1 + i * 2);
+			lua_rawgeti(L, 1, 2 + i * 2);
+			float x = (float)luaL_checknumber(L, -2);
+			float y = (float)luaL_checknumber(L, -1);
+			vecs[i] = Physics::scaleDown(b2Vec2(x, y));
+			lua_pop(L, 2);
+		}
 	}
+	else
+	{
+		for (int i = 0; i < vcount; i++)
+		{
+			float x = (float)luaL_checknumber(L, 1 + i * 2);
+			float y = (float)luaL_checknumber(L, 2 + i * 2);
+			vecs[i] = Physics::scaleDown(b2Vec2(x, y));
+		}
+	}
+
+	b2PolygonShape *s = new b2PolygonShape();
 
 	try
 	{
@@ -125,30 +146,50 @@ int Physics::newPolygonShape(lua_State *L)
 	}
 
 	PolygonShape *p = new PolygonShape(s);
-
-	luax_pushtype(L, "PolygonShape", PHYSICS_POLYGON_SHAPE_T, p);
+	luax_pushtype(L, p);
+	p->release();
 	return 1;
 }
 
 int Physics::newChainShape(lua_State *L)
 {
 	int argc = lua_gettop(L)-1; // first argument is looping
-	int vcount = (int)argc/2;
 
-	b2ChainShape *s = new b2ChainShape();
+	bool istable = lua_istable(L, 2);
 
-	bool loop = luax_toboolean(L, 1);
+	if (istable)
+		argc = (int) luax_objlen(L, 2);
 
+	if (argc % 2 != 0)
+		return luaL_error(L, "Number of vertex components must be a multiple of two.");
+
+	int vcount = argc/2;
+	bool loop = luax_checkboolean(L, 1);
 	b2Vec2 *vecs = new b2Vec2[vcount];
 
-	for (int i = 0; i<vcount; i++)
+	if (istable)
 	{
-		float x = (float)lua_tonumber(L, -2);
-		float y = (float)lua_tonumber(L, -1);
-		vecs[i].Set(x, y);
-		vecs[i] = Physics::scaleDown(vecs[i]);
-		lua_pop(L, 2);
+		for (int i = 0; i < vcount; i++)
+		{
+			lua_rawgeti(L, 2, 1 + i * 2);
+			lua_rawgeti(L, 2, 2 + i * 2);
+			float x = (float)lua_tonumber(L, -2);
+			float y = (float)lua_tonumber(L, -1);
+			vecs[i] = Physics::scaleDown(b2Vec2(x, y));
+			lua_pop(L, 2);
+		}
 	}
+	else
+	{
+		for (int i = 0; i < vcount; i++)
+		{
+			float x = (float)luaL_checknumber(L, 2 + i * 2);
+			float y = (float)luaL_checknumber(L, 3 + i * 2);
+			vecs[i] = Physics::scaleDown(b2Vec2(x, y));
+		}
+	}
+
+	b2ChainShape *s = new b2ChainShape();
 
 	try
 	{
@@ -160,14 +201,15 @@ int Physics::newChainShape(lua_State *L)
 	catch (love::Exception &)
 	{
 		delete[] vecs;
+		delete s;
 		throw;
 	}
 
 	delete[] vecs;
 
 	ChainShape *c = new ChainShape(s);
-
-	luax_pushtype(L, "ChainShape", PHYSICS_CHAIN_SHAPE_T, c);
+	luax_pushtype(L, c);
+	c->release();
 	return 1;
 }
 
@@ -181,9 +223,14 @@ MouseJoint *Physics::newMouseJoint(Body *body, float x, float y)
 	return new MouseJoint(body, x, y);
 }
 
-RevoluteJoint *Physics::newRevoluteJoint(Body *body1, Body *body2, float x, float y, bool collideConnected)
+RevoluteJoint *Physics::newRevoluteJoint(Body *body1, Body *body2, float xA, float yA, float xB, float yB, bool collideConnected)
 {
-	return new RevoluteJoint(body1, body2, x, y, collideConnected);
+	return new RevoluteJoint(body1, body2, xA, yA, xB, yB, collideConnected);
+}
+
+RevoluteJoint *Physics::newRevoluteJoint(Body *body1, Body *body2, float xA, float yA, float xB, float yB, bool collideConnected, float referenceAngle)
+{
+	return new RevoluteJoint(body1, body2, xA, yA, xB, yB, collideConnected, referenceAngle);
 }
 
 PrismaticJoint *Physics::newPrismaticJoint(Body *body1, Body *body2, float xA, float yA, float xB, float yB, float ax, float ay, bool collideConnected)
@@ -191,6 +238,11 @@ PrismaticJoint *Physics::newPrismaticJoint(Body *body1, Body *body2, float xA, f
 	return new PrismaticJoint(body1, body2, xA, yA, xB, yB, ax, ay, collideConnected);
 }
 
+PrismaticJoint *Physics::newPrismaticJoint(Body *body1, Body *body2, float xA, float yA, float xB, float yB, float ax, float ay, bool collideConnected, float referenceAngle)
+{
+	return new PrismaticJoint(body1, body2, xA, yA, xB, yB, ax, ay, collideConnected, referenceAngle);
+}
+
 PulleyJoint *Physics::newPulleyJoint(Body *body1, Body *body2, b2Vec2 groundAnchor1, b2Vec2 groundAnchor2, b2Vec2 anchor1, b2Vec2 anchor2, float ratio, bool collideConnected)
 {
 	return new PulleyJoint(body1, body2, groundAnchor1, groundAnchor2, anchor1, anchor2, ratio, collideConnected);
@@ -211,6 +263,11 @@ WeldJoint *Physics::newWeldJoint(Body *body1, Body *body2, float xA, float yA, f
 	return new WeldJoint(body1, body2, xA, yA, xB, yB, collideConnected);
 }
 
+WeldJoint *Physics::newWeldJoint(Body *body1, Body *body2, float xA, float yA, float xB, float yB, bool collideConnected, float referenceAngle)
+{
+	return new WeldJoint(body1, body2, xA, yA, xB, yB, collideConnected, referenceAngle);
+}
+
 WheelJoint *Physics::newWheelJoint(Body *body1, Body *body2, float xA, float yA, float xB, float yB, float ax, float ay, bool collideConnected)
 {
 	return new WheelJoint(body1, body2, xA, yA, xB, yB, ax, ay, collideConnected);
@@ -226,9 +283,9 @@ MotorJoint *Physics::newMotorJoint(Body *body1, Body *body2)
 	return new MotorJoint(body1, body2);
 }
 
-MotorJoint *Physics::newMotorJoint(Body *body1, Body *body2, float correctionFactor)
+MotorJoint *Physics::newMotorJoint(Body *body1, Body *body2, float correctionFactor, bool collideConnected)
 {
-	return new MotorJoint(body1, body2, correctionFactor);
+	return new MotorJoint(body1, body2, correctionFactor, collideConnected);
 }
 
 
@@ -239,15 +296,15 @@ Fixture *Physics::newFixture(Body *body, Shape *shape, float density)
 
 int Physics::getDistance(lua_State *L)
 {
-	Fixture *fixtureA = luax_checktype<Fixture>(L, 1, "Fixture", PHYSICS_FIXTURE_T);
-	Fixture *fixtureB = luax_checktype<Fixture>(L, 2, "Fixture", PHYSICS_FIXTURE_T);
+	Fixture *fixtureA = luax_checktype<Fixture>(L, 1);
+	Fixture *fixtureB = luax_checktype<Fixture>(L, 2);
 	b2DistanceProxy pA, pB;
 	b2DistanceInput i;
 	b2DistanceOutput o;
 	b2SimplexCache c;
 	c.count = 0;
 
-	EXCEPT_GUARD(
+	luax_catchexcept(L, [&]() {
 		pA.Set(fixtureA->fixture->GetShape(), 0);
 		pB.Set(fixtureB->fixture->GetShape(), 0);
 		i.proxyA = pA;
@@ -256,7 +313,7 @@ int Physics::getDistance(lua_State *L)
 		i.transformB = fixtureB->fixture->GetBody()->GetTransform();
 		i.useRadii = true;
 		b2Distance(&o, &c, &i);
-	)
+	});
 
 	lua_pushnumber(L, Physics::scaleUp(o.distance));
 	lua_pushnumber(L, Physics::scaleUp(o.pointA.x));
@@ -266,37 +323,37 @@ int Physics::getDistance(lua_State *L)
 	return 5;
 }
 
-void Physics::setMeter(int scale)
+void Physics::setMeter(float scale)
 {
 	if (scale < 1) throw love::Exception("Physics error: invalid meter");
 	Physics::meter = scale;
 }
 
-int Physics::getMeter()
+float Physics::getMeter()
 {
 	return meter;
 }
 
 void Physics::scaleDown(float &x, float &y)
 {
-	x /= (float)meter;
-	y /= (float)meter;
+	x /= meter;
+	y /= meter;
 }
 
 void Physics::scaleUp(float &x, float &y)
 {
-	x *= (float)meter;
-	y *= (float)meter;
+	x *= meter;
+	y *= meter;
 }
 
 float Physics::scaleDown(float f)
 {
-	return f/(float)meter;
+	return f/meter;
 }
 
 float Physics::scaleUp(float f)
 {
-	return f*(float)meter;
+	return f*meter;
 }
 
 b2Vec2 Physics::scaleDown(const b2Vec2 &v)
