@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -32,23 +32,39 @@ namespace physics
 namespace box2d
 {
 
+love::Type PrismaticJoint::type("PrismaticJoint", &Joint::type);
+
 PrismaticJoint::PrismaticJoint(Body *body1, Body *body2, float xA, float yA, float xB, float yB, float ax, float ay, bool collideConnected)
 	: Joint(body1, body2)
 	, joint(NULL)
 {
 	b2PrismaticJointDef def;
+	init(def, body1, body2, xA, yA, xB, yB, ax, ay, collideConnected);
+	joint = (b2PrismaticJoint *)createJoint(&def);
+}
 
+PrismaticJoint::PrismaticJoint(Body *body1, Body *body2, float xA, float yA, float xB, float yB, float ax, float ay, bool collideConnected, float referenceAngle)
+	: Joint(body1, body2)
+	, joint(NULL)
+{
+	b2PrismaticJointDef def;
+	init(def, body1, body2, xA, yA, xB, yB, ax, ay, collideConnected);
+	def.referenceAngle = referenceAngle;
+	joint = (b2PrismaticJoint *)createJoint(&def);
+}
+
+PrismaticJoint::~PrismaticJoint()
+{
+}
+
+void PrismaticJoint::init(b2PrismaticJointDef &def, Body *body1, Body *body2, float xA, float yA, float xB, float yB, float ax, float ay, bool collideConnected)
+{
 	def.Initialize(body1->body, body2->body, Physics::scaleDown(b2Vec2(xA,yA)), b2Vec2(ax,ay));
 	def.localAnchorB = body2->body->GetLocalPoint(Physics::scaleDown(b2Vec2(xB, yB)));
 	def.lowerTranslation = 0.0f;
 	def.upperTranslation = 100.0f;
 	def.enableLimit = true;
 	def.collideConnected = collideConnected;
-	joint = (b2PrismaticJoint *)createJoint(&def);
-}
-
-PrismaticJoint::~PrismaticJoint()
-{
 }
 
 float PrismaticJoint::getJointTranslation() const
@@ -101,7 +117,7 @@ void PrismaticJoint::setLimitsEnabled(bool enable)
 	joint->EnableLimit(enable);
 }
 
-bool PrismaticJoint::hasLimitsEnabled() const
+bool PrismaticJoint::areLimitsEnabled() const
 {
 	return joint->IsLimitEnabled();
 }
@@ -138,6 +154,19 @@ int PrismaticJoint::getLimits(lua_State *L)
 	return 2;
 }
 
+int PrismaticJoint::getAxis(lua_State *L)
+{
+	b2Vec2 axis = joint->GetLocalAxisA();
+	getBodyA()->getWorldVector(axis.x, axis.y, axis.x, axis.y);
+	lua_pushnumber(L, axis.x);
+	lua_pushnumber(L, axis.y);
+	return 2;
+}
+
+float PrismaticJoint::getReferenceAngle() const
+{
+	return joint->GetReferenceAngle();
+}
 
 } // box2d
 } // physics
