@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2015 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -39,11 +39,15 @@ class PrismaticJoint : public Joint
 {
 public:
 
+	static love::Type type;
+
 	/**
 	 * Creates a new PrismaticJoint connecting body1 and body2.
 	 **/
 	PrismaticJoint(Body *body1, Body *body2, float xA, float yA, float xB, float yB, float ax, float ay, bool collideConnected);
 
+	PrismaticJoint(Body *body1, Body *body2, float xA, float yA, float xB, float yB, float ax, float ay, bool collideConnected, float referenceAngle);
+
 	virtual ~PrismaticJoint();
 
 	/**
@@ -100,7 +104,7 @@ public:
 	/**
 	 * Checks whether limits are enabled.
 	 **/
-	bool hasLimitsEnabled() const;
+	bool areLimitsEnabled() const;
 
 	/**
 	 * Sets the upper limit, usually in meters.
@@ -134,10 +138,24 @@ public:
 	 **/
 	int getLimits(lua_State *L);
 
+	/**
+	 * Gets the axis unit vector, relative to body1.
+	 * @returns The X component of the axis unit vector.
+	 * @returns The Y component of the axis unit vector.
+	 **/
+	int getAxis(lua_State *L);
+
+	/**
+	 * Gets the reference angle.
+	 **/
+	float getReferenceAngle() const;
+
 private:
 
 	// The Box2D prismatic joint object.
 	b2PrismaticJoint *joint;
+
+	void init(b2PrismaticJointDef &def, Body *body1, Body *body2, float xA, float yA, float xB, float yB, float ax, float ay, bool collideConnected);
 };
 
 } // box2d
