@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -39,10 +39,14 @@ class RevoluteJoint : public Joint
 {
 public:
 
+	static love::Type type;
+
 	/**
 	 * Creates a new RevoluteJoint connecting body1 and body2.
 	 **/
-	RevoluteJoint(Body *body1, Body *body2, float x, float y, bool collideConnected);
+	RevoluteJoint(Body *body1, Body *body2, float xA, float yA, float xB, float yB, bool collideConnected);
+
+	RevoluteJoint(Body *body1, Body *body2, float xA, float yA, float xB, float yB, bool collideConnected, float referenceAngle);
 
 	virtual ~RevoluteJoint();
 
@@ -100,7 +104,7 @@ public:
 	/**
 	 * Checks whether limits are enabled.
 	 **/
-	bool hasLimitsEnabled() const;
+	bool areLimitsEnabled() const;
 
 	/**
 	 * Sets the upper limit in degrees.
@@ -134,10 +138,17 @@ public:
 	 **/
 	int getLimits(lua_State *L);
 
+	/**
+	 * Gets the reference angle.
+	 **/
+	float getReferenceAngle() const;
+
 private:
 
 	// The Box2D revolute joint object.
 	b2RevoluteJoint *joint;
+
+	void init(b2RevoluteJointDef &def, Body *body1, Body *body2, float xA, float yA, float xB, float yB, bool collideConnected);
 };
 
 } // box2d
