@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -34,14 +34,24 @@ namespace physics
 namespace box2d
 {
 
+love::Type WeldJoint::type("WeldJoint", &Joint::type);
+
 WeldJoint::WeldJoint(Body *body1, Body *body2, float xA, float yA, float xB, float yB, bool collideConnected)
 	: Joint(body1, body2)
 	, joint(NULL)
 {
 	b2WeldJointDef def;
-	def.Initialize(body1->body, body2->body, Physics::scaleDown(b2Vec2(xA,yA)));
-	def.localAnchorB = body2->body->GetLocalPoint(Physics::scaleDown(b2Vec2(xB, yB)));
-	def.collideConnected = collideConnected;
+	init(def, body1, body2, xA, yA, xB, yB, collideConnected);
+	joint = (b2WeldJoint *)createJoint(&def);
+}
+
+WeldJoint::WeldJoint(Body *body1, Body *body2, float xA, float yA, float xB, float yB, bool collideConnected, float referenceAngle)
+	: Joint(body1, body2)
+	, joint(NULL)
+{
+	b2WeldJointDef def;
+	init(def, body1, body2, xA, yA, xB, yB, collideConnected);
+	def.referenceAngle = referenceAngle;
 	joint = (b2WeldJoint *)createJoint(&def);
 }
 
@@ -49,6 +59,13 @@ WeldJoint::~WeldJoint()
 {
 }
 
+void WeldJoint::init(b2WeldJointDef &def, Body *body1, Body *body2, float xA, float yA, float xB, float yB, bool collideConnected)
+{
+	def.Initialize(body1->body, body2->body, Physics::scaleDown(b2Vec2(xA,yA)));
+	def.localAnchorB = body2->body->GetLocalPoint(Physics::scaleDown(b2Vec2(xB, yB)));
+	def.collideConnected = collideConnected;
+}
+
 void WeldJoint::setFrequency(float hz)
 {
 	joint->SetFrequency(hz);
@@ -69,6 +86,11 @@ float WeldJoint::getDampingRatio() const
 	return joint->GetDampingRatio();
 }
 
+float WeldJoint::getReferenceAngle() const
+{
+	return joint->GetReferenceAngle();
+}
+
 } // box2d
 } // physics
 } // love
