@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -32,6 +32,8 @@ namespace physics
 namespace box2d
 {
 
+love::Type WheelJoint::type("WheelJoint", &Joint::type);
+
 WheelJoint::WheelJoint(Body *body1, Body *body2, float xA, float yA, float xB, float yB, float ax, float ay, bool collideConnected)
 	: Joint(body1, body2)
 	, joint(NULL)
@@ -113,6 +115,14 @@ float WheelJoint::getSpringDampingRatio() const
 	return joint->GetSpringDampingRatio();
 }
 
+int WheelJoint::getAxis(lua_State *L)
+{
+	b2Vec2 axis = joint->GetLocalAxisA();
+	getBodyA()->getWorldVector(axis.x, axis.y, axis.x, axis.y);
+	lua_pushnumber(L, axis.x);
+	lua_pushnumber(L, axis.y);
+	return 2;
+}
 
 } // box2d
 } // physics
