@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -66,10 +66,13 @@ public:
 	friend class Body;
 	friend class Fixture;
 
+	static love::Type type;
+
 	class ContactCallback
 	{
 	public:
 		Reference *ref;
+		lua_State *L;
 		ContactCallback();
 		~ContactCallback();
 		void process(b2Contact *contact, const b2ContactImpulse *impulse = NULL);
@@ -79,6 +82,7 @@ public:
 	{
 	public:
 		Reference *ref;
+		lua_State *L;
 		ContactFilter();
 		~ContactFilter();
 		bool process(Fixture *a, Fixture *b);
@@ -87,19 +91,23 @@ public:
 	class QueryCallback : public b2QueryCallback
 	{
 	public:
-		Reference *ref;
-		QueryCallback();
+		QueryCallback(lua_State *L, int idx);
 		~QueryCallback();
 		virtual bool ReportFixture(b2Fixture *fixture);
+	private:
+		lua_State *L;
+		int funcidx;
 	};
 
 	class RayCastCallback : public b2RayCastCallback
 	{
 	public:
-		Reference *ref;
-		RayCastCallback();
+		RayCastCallback(lua_State *L, int idx);
 		~RayCastCallback();
 		virtual float32 ReportFixture(b2Fixture *fixture, const b2Vec2 &point, const b2Vec2 &normal, float32 fraction);
+	private:
+		lua_State *L;
+		int funcidx;
 	};
 
 	/**
@@ -125,6 +133,7 @@ public:
 	 * @param dt The timestep.
 	 **/
 	void update(float dt);
+	void update(float dt, int velocityIterations, int positionIterations);
 
 	// From b2ContactListener
 	void BeginContact(b2Contact *contact);
@@ -157,6 +166,13 @@ public:
 	 **/
 	int getCallbacks(lua_State *L);
 
+	/**
+	 * Updates the Lua thread/coroutine used when callbacks are executed in
+	 * the update method. This should be called in the same Lua function which
+	 * calls update().
+	 **/
+	void setCallbacksL(lua_State *L);
+
 	/**
 	 * Sets the ContactFilter callback.
 	 **/
@@ -229,19 +245,19 @@ public:
 	 * Get an array of all the Bodies in the World.
 	 * @return An array of Bodies.
 	 **/
-	int getBodyList(lua_State *L) const;
+	int getBodies(lua_State *L) const;
 
 	/**
 	 * Get an array of all the Joints in the World.
 	 * @return An array of Joints.
 	 **/
-	int getJointList(lua_State *L) const;
+	int getJoints(lua_State *L) const;
 
 	/**
 	 * Get an array of all the Contacts in the World.
 	 * @return An array of Contacts.
 	 **/
-	int getContactList(lua_State *L) const;
+	int getContacts(lua_State *L) const;
 
 	/**
 	 * Gets the ground body.
@@ -281,8 +297,6 @@ private:
 	// Contact callbacks.
 	ContactCallback begin, end, presolve, postsolve;
 	ContactFilter filter;
-	QueryCallback query;
-	RayCastCallback	raycast;
 };
 
 } // box2d
