@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2015 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -29,7 +29,7 @@ namespace box2d
 
 CircleShape *luax_checkcircleshape(lua_State *L, int idx)
 {
-	return luax_checktype<CircleShape>(L, idx, "CircleShape", PHYSICS_CIRCLE_SHAPE_T);
+	return luax_checktype<CircleShape>(L, idx);
 }
 
 int w_CircleShape_getRadius(lua_State *L)
@@ -64,26 +64,18 @@ int w_CircleShape_setPoint(lua_State *L)
 	return 0;
 }
 
-static const luaL_Reg functions[] =
+static const luaL_Reg w_CircleShape_functions[] =
 {
 	{ "getRadius", w_CircleShape_getRadius },
 	{ "setRadius", w_CircleShape_setRadius },
 	{ "getPoint", w_CircleShape_getPoint },
 	{ "setPoint", w_CircleShape_setPoint },
-	// From Shape.
-	{ "getType", w_Shape_getType },
-	{ "getRadius", w_Shape_getRadius },
-	{ "getChildCount", w_Shape_getChildCount },
-	{ "testPoint", w_Shape_testPoint },
-	{ "rayCast", w_Shape_rayCast },
-	{ "computeAABB", w_Shape_computeAABB },
-	{ "computeMass", w_Shape_computeMass },
 	{ 0, 0 }
 };
 
 extern "C" int luaopen_circleshape(lua_State *L)
 {
-	return luax_register_type(L, "CircleShape", functions);
+	return luax_register_type(L, &CircleShape::type, w_Shape_functions, w_CircleShape_functions, nullptr);
 }
 
 } // box2d
