@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2015 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -34,12 +34,6 @@ namespace box2d
 {
 
 DistanceJoint *luax_checkdistancejoint(lua_State *L, int idx);
-int w_DistanceJoint_setLength(lua_State *L);
-int w_DistanceJoint_getLength(lua_State *L);
-int w_DistanceJoint_setFrequency(lua_State *L);
-int w_DistanceJoint_getFrequency(lua_State *L);
-int w_DistanceJoint_setDampingRatio(lua_State *L);
-int w_DistanceJoint_getDampingRatio(lua_State *L);
 extern "C" int luaopen_distancejoint(lua_State *L);
 
 } // box2d
