@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2015 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -29,7 +29,61 @@ namespace box2d
 
 EdgeShape *luax_checkedgeshape(lua_State *L, int idx)
 {
-	return luax_checktype<EdgeShape>(L, idx, "EdgeShape", PHYSICS_EDGE_SHAPE_T);
+	return luax_checktype<EdgeShape>(L, idx);
+}
+
+int w_EdgeShape_setNextVertex(lua_State *L)
+{
+	EdgeShape *t = luax_checkedgeshape(L, 1);
+	if (lua_isnoneornil(L, 2))
+		t->setNextVertex();
+	else
+	{
+		float x = (float)luaL_checknumber(L, 2);
+		float y = (float)luaL_checknumber(L, 3);
+		t->setNextVertex(x, y);
+	}
+	return 0;
+}
+
+int w_EdgeShape_setPreviousVertex(lua_State *L)
+{
+	EdgeShape *t = luax_checkedgeshape(L, 1);
+	if (lua_isnoneornil(L, 2))
+		t->setPreviousVertex();
+	else
+	{
+		float x = (float)luaL_checknumber(L, 2);
+		float y = (float)luaL_checknumber(L, 3);
+		t->setPreviousVertex(x, y);
+	}
+	return 0;
+}
+
+int w_EdgeShape_getNextVertex(lua_State *L)
+{
+	EdgeShape *t = luax_checkedgeshape(L, 1);
+	float x, y;
+	if (t->getNextVertex(x, y))
+	{
+		lua_pushnumber(L, x);
+		lua_pushnumber(L, y);
+		return 2;
+	}
+	return 0;
+}
+
+int w_EdgeShape_getPreviousVertex(lua_State *L)
+{
+	EdgeShape *t = luax_checkedgeshape(L, 1);
+	float x, y;
+	if (t->getPreviousVertex(x, y))
+	{
+		lua_pushnumber(L, x);
+		lua_pushnumber(L, y);
+		return 2;
+	}
+	return 0;
 }
 
 int w_EdgeShape_getPoints(lua_State *L)
@@ -39,23 +93,19 @@ int w_EdgeShape_getPoints(lua_State *L)
 	return t->getPoints(L);
 }
 
-static const luaL_Reg functions[] =
+static const luaL_Reg w_EdgeShape_functions[] =
 {
+	{ "setNextVertex", w_EdgeShape_setNextVertex },
+	{ "setPreviousVertex", w_EdgeShape_setPreviousVertex },
+	{ "getNextVertex", w_EdgeShape_getNextVertex },
+	{ "getPreviousVertex", w_EdgeShape_getPreviousVertex },
 	{ "getPoints", w_EdgeShape_getPoints },
-	// From Shape.
-	{ "getType", w_Shape_getType },
-	{ "getRadius", w_Shape_getRadius },
-	{ "getChildCount", w_Shape_getChildCount },
-	{ "testPoint", w_Shape_testPoint },
-	{ "rayCast", w_Shape_rayCast },
-	{ "computeAABB", w_Shape_computeAABB },
-	{ "computeMass", w_Shape_computeMass },
 	{ 0, 0 }
 };
 
 extern "C" int luaopen_edgeshape(lua_State *L)
 {
-	return luax_register_type(L, "EdgeShape", functions);
+	return luax_register_type(L, &EdgeShape::type, w_Shape_functions, w_EdgeShape_functions, nullptr);
 }
 
 } // box2d
