@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -30,7 +30,7 @@ namespace box2d
 
 Fixture *luax_checkfixture(lua_State *L, int idx)
 {
-	Fixture *f = luax_checktype<Fixture>(L, idx, "Fixture", PHYSICS_FIXTURE_T);
+	Fixture *f = luax_checktype<Fixture>(L, idx);
 	if (!f->isValid())
 		luaL_error(L, "Attempt to use destroyed fixture.");
 	return f;
@@ -65,14 +65,14 @@ int w_Fixture_setDensity(lua_State *L)
 {
 	Fixture *t = luax_checkfixture(L, 1);
 	float arg1 = (float)luaL_checknumber(L, 2);
-	EXCEPT_GUARD(t->setDensity(arg1);)
+	luax_catchexcept(L, [&](){ t->setDensity(arg1); });
 	return 0;
 }
 
 int w_Fixture_setSensor(lua_State *L)
 {
 	Fixture *t = luax_checkfixture(L, 1);
-	bool arg1 = luax_toboolean(L, 2);
+	bool arg1 = luax_checkboolean(L, 2);
 	t->setSensor(arg1);
 	return 0;
 }
@@ -111,33 +111,32 @@ int w_Fixture_getBody(lua_State *L)
 	Body *body = t->getBody();
 	if (body == 0)
 		return 0;
-	body->retain();
-	luax_pushtype(L, "Body", PHYSICS_BODY_T, body);
+	luax_pushtype(L, body);
 	return 1;
 }
 
 int w_Fixture_getShape(lua_State *L)
 {
 	Fixture *t = luax_checkfixture(L, 1);
-	Shape *shape = t->getShape();
-	if (shape == 0)
+	Shape * shape = t->getShape();
+	if (shape == nullptr)
 		return 0;
 	switch (shape->getType())
 	{
 	case Shape::SHAPE_EDGE:
-		luax_pushtype(L, "EdgeShape", PHYSICS_EDGE_SHAPE_T, shape);
+		luax_pushtype(L, dynamic_cast<EdgeShape *>(shape));
 		break;
 	case Shape::SHAPE_CHAIN:
-		luax_pushtype(L, "ChainShape", PHYSICS_CHAIN_SHAPE_T, shape);
+		luax_pushtype(L, dynamic_cast<ChainShape *>(shape));
 		break;
 	case Shape::SHAPE_CIRCLE:
-		luax_pushtype(L, "CircleShape", PHYSICS_CIRCLE_SHAPE_T, shape);
+		luax_pushtype(L, dynamic_cast<CircleShape *>(shape));
 		break;
 	case Shape::SHAPE_POLYGON:
-		luax_pushtype(L, "PolygonShape", PHYSICS_POLYGON_SHAPE_T, shape);
+		luax_pushtype(L, dynamic_cast<PolygonShape *>(shape));
 		break;
 	default:
-		luax_pushtype(L, "Shape", PHYSICS_SHAPE_T, shape);
+		luax_pushtype(L, shape);
 		break;
 	}
 	return 1;
@@ -157,7 +156,7 @@ int w_Fixture_rayCast(lua_State *L)
 	Fixture *t = luax_checkfixture(L, 1);
 	lua_remove(L, 1);
 	int ret = 0;
-	EXCEPT_GUARD(ret = t->rayCast(L);)
+	luax_catchexcept(L, [&](){ ret = t->rayCast(L); });
 	return ret;
 }
 
@@ -165,9 +164,9 @@ int w_Fixture_setFilterData(lua_State *L)
 {
 	Fixture *t = luax_checkfixture(L, 1);
 	int v[3];
-	v[0] = luaL_checkint(L, 2);
-	v[1] = luaL_checkint(L, 3);
-	v[2] = luaL_checkint(L, 4);
+	v[0] = (int) luaL_checkinteger(L, 2);
+	v[1] = (int) luaL_checkinteger(L, 3);
+	v[2] = (int) luaL_checkinteger(L, 4);
 	t->setFilterData(v);
 	return 0;
 }
@@ -250,7 +249,7 @@ int w_Fixture_getGroupIndex(lua_State *L)
 int w_Fixture_setGroupIndex(lua_State *L)
 {
 	Fixture *t = luax_checkfixture(L, 1);
-	int i = luaL_checkint(L, 2);
+	int i = (int) luaL_checkinteger(L, 2);
 	t->setGroupIndex(i);
 	return 0;
 }
@@ -258,11 +257,18 @@ int w_Fixture_setGroupIndex(lua_State *L)
 int w_Fixture_destroy(lua_State *L)
 {
 	Fixture *t = luax_checkfixture(L, 1);
-	EXCEPT_GUARD(t->destroy();)
+	luax_catchexcept(L, [&](){ t->destroy(); });
 	return 0;
 }
 
-static const luaL_Reg functions[] =
+int w_Fixture_isDestroyed(lua_State *L)
+{
+	Fixture *f = luax_checktype<Fixture>(L, 1);
+	luax_pushboolean(L, !f->isValid());
+	return 1;
+}
+
+static const luaL_Reg w_Fixture_functions[] =
 {
 	{ "getType", w_Fixture_getType },
 	{ "setFriction", w_Fixture_setFriction },
@@ -290,12 +296,13 @@ static const luaL_Reg functions[] =
 	{ "getGroupIndex", w_Fixture_getGroupIndex },
 	{ "setGroupIndex", w_Fixture_setGroupIndex },
 	{ "destroy", w_Fixture_destroy },
+	{ "isDestroyed", w_Fixture_isDestroyed },
 	{ 0, 0 }
 };
 
 extern "C" int luaopen_fixture(lua_State *L)
 {
-	return luax_register_type(L, "Fixture", functions);
+	return luax_register_type(L, &Fixture::type, w_Fixture_functions, nullptr);
 }
 
 } // box2d
