@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -30,7 +30,7 @@ namespace box2d
 
 GearJoint *luax_checkgearjoint(lua_State *L, int idx)
 {
-	GearJoint *j = luax_checktype<GearJoint>(L, idx, "GearJoint", PHYSICS_GEAR_JOINT_T);
+	GearJoint *j = luax_checktype<GearJoint>(L, idx);
 	if (!j->isValid())
 		luaL_error(L, "Attempt to use destroyed joint.");
 	return j;
@@ -40,7 +40,7 @@ int w_GearJoint_setRatio(lua_State *L)
 {
 	GearJoint *t = luax_checkgearjoint(L, 1);
 	float arg1 = (float)luaL_checknumber(L, 2);
-	EXCEPT_GUARD(t->setRatio(arg1);)
+	luax_catchexcept(L, [&](){ t->setRatio(arg1); });
 	return 0;
 }
 
@@ -51,23 +51,33 @@ int w_GearJoint_getRatio(lua_State *L)
 	return 1;
 }
 
-static const luaL_Reg functions[] =
+int w_GearJoint_getJoints(lua_State *L)
+{
+	GearJoint *t = luax_checkgearjoint(L, 1);
+	Joint *j1 = nullptr;
+	Joint *j2 = nullptr;
+
+	luax_catchexcept(L, [&]() {
+		j1 = t->getJointA();
+		j2 = t->getJointB();
+	});
+
+	luax_pushjoint(L, j1);
+	luax_pushjoint(L, j2);
+	return 2;
+}
+
+static const luaL_Reg w_GearJoint_functions[] =
 {
 	{ "setRatio", w_GearJoint_setRatio },
 	{ "getRatio", w_GearJoint_getRatio },
-	// From Joint.
-	{ "getType", w_Joint_getType },
-	{ "getAnchors", w_Joint_getAnchors },
-	{ "getReactionForce", w_Joint_getReactionForce },
-	{ "getReactionTorque", w_Joint_getReactionTorque },
-	{ "getCollideConnected", w_Joint_getCollideConnected },
-	{ "destroy", w_Joint_destroy },
+	{ "getJoints", w_GearJoint_getJoints },
 	{ 0, 0 }
 };
 
 extern "C" int luaopen_gearjoint(lua_State *L)
 {
-	return luax_register_type(L, "GearJoint", functions);
+	return luax_register_type(L, &GearJoint::type, w_Joint_functions, w_GearJoint_functions, nullptr);
 }
 
 } // box2d
