@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -29,7 +29,7 @@ namespace box2d
 
 MouseJoint *luax_checkmousejoint(lua_State *L, int idx)
 {
-	MouseJoint *j = luax_checktype<MouseJoint>(L, idx, "MouseJoint", PHYSICS_MOUSE_JOINT_T);
+	MouseJoint *j = luax_checktype<MouseJoint>(L, idx);
 	if (!j->isValid())
 		luaL_error(L, "Attempt to use destroyed joint.");
 	return j;
@@ -70,7 +70,7 @@ int w_MouseJoint_setFrequency(lua_State *L)
 {
 	MouseJoint *t = luax_checkmousejoint(L, 1);
 	float arg1 = (float)luaL_checknumber(L, 2);
-	t->setFrequency(arg1);
+	luax_catchexcept(L, [&]() { t->setFrequency(arg1); });
 	return 0;
 }
 
@@ -96,7 +96,7 @@ int w_MouseJoint_getDampingRatio(lua_State *L)
 	return 1;
 }
 
-static const luaL_Reg functions[] =
+static const luaL_Reg w_MouseJoint_functions[] =
 {
 	{ "setTarget", w_MouseJoint_setTarget },
 	{ "getTarget", w_MouseJoint_getTarget },
@@ -106,19 +106,12 @@ static const luaL_Reg functions[] =
 	{ "getFrequency", w_MouseJoint_getFrequency },
 	{ "setDampingRatio", w_MouseJoint_setDampingRatio },
 	{ "getDampingRatio", w_MouseJoint_getDampingRatio },
-	// From Joint.
-	{ "getType", w_Joint_getType },
-	{ "getAnchors", w_Joint_getAnchors },
-	{ "getReactionForce", w_Joint_getReactionForce },
-	{ "getReactionTorque", w_Joint_getReactionTorque },
-	{ "getCollideConnected", w_Joint_getCollideConnected },
-	{ "destroy", w_Joint_destroy },
 	{ 0, 0 }
 };
 
 extern "C" int luaopen_mousejoint(lua_State *L)
 {
-	return luax_register_type(L, "MouseJoint", functions);
+	return luax_register_type(L, &MouseJoint::type, w_Joint_functions, w_MouseJoint_functions, nullptr);
 }
 
 } // box2d
