@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -30,7 +30,7 @@ namespace box2d
 
 Shape *luax_checkshape(lua_State *L, int idx)
 {
-	return luax_checktype<Shape>(L, idx, "Shape", PHYSICS_SHAPE_T);
+	return luax_checktype<Shape>(L, idx);
 }
 
 int w_Shape_getType(lua_State *L)
@@ -76,7 +76,7 @@ int w_Shape_rayCast(lua_State *L)
 	Shape *t = luax_checkshape(L, 1);
 	lua_remove(L, 1);
 	int ret = 0;
-	EXCEPT_GUARD(ret = t->rayCast(L);)
+	luax_catchexcept(L, [&](){ ret = t->rayCast(L); });
 	return ret;
 }
 
@@ -94,7 +94,7 @@ int w_Shape_computeMass(lua_State *L)
 	return t->computeMass(L);
 }
 
-static const luaL_Reg functions[] =
+const luaL_Reg w_Shape_functions[] =
 {
 	{ "getType", w_Shape_getType },
 	{ "getRadius", w_Shape_getRadius },
@@ -108,7 +108,7 @@ static const luaL_Reg functions[] =
 
 extern "C" int luaopen_shape(lua_State *L)
 {
-	return luax_register_type(L, "Shape", functions);
+	return luax_register_type(L, &Shape::type, w_Shape_functions, nullptr);
 }
 
 } // box2d
