@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -29,7 +29,7 @@ namespace box2d
 
 WheelJoint *luax_checkwheeljoint(lua_State *L, int idx)
 {
-	WheelJoint *j = luax_checktype<WheelJoint>(L, idx, "WheelJoint", PHYSICS_WHEEL_JOINT_T);
+	WheelJoint *j = luax_checktype<WheelJoint>(L, idx);
 	if (!j->isValid())
 		luaL_error(L, "Attempt to use destroyed joint.");
 	return j;
@@ -52,7 +52,7 @@ int w_WheelJoint_getJointSpeed(lua_State *L)
 int w_WheelJoint_setMotorEnabled(lua_State *L)
 {
 	WheelJoint *t = luax_checkwheeljoint(L, 1);
-	bool arg1 = luax_toboolean(L, 2);
+	bool arg1 = luax_checkboolean(L, 2);
 	t->setMotorEnabled(arg1);
 	return 0;
 }
@@ -132,7 +132,14 @@ int w_WheelJoint_getSpringDampingRatio(lua_State *L)
 	return 1;
 }
 
-static const luaL_Reg functions[] =
+int w_WheelJoint_getAxis(lua_State *L)
+{
+	WheelJoint *t = luax_checkwheeljoint(L, 1);
+	lua_remove(L, 1);
+	return t->getAxis(L);
+}
+
+static const luaL_Reg w_WheelJoint_functions[] =
 {
 	{ "getJointTranslation", w_WheelJoint_getJointTranslation },
 	{ "getJointSpeed", w_WheelJoint_getJointSpeed },
@@ -147,19 +154,13 @@ static const luaL_Reg functions[] =
 	{ "getSpringFrequency", w_WheelJoint_getSpringFrequency },
 	{ "setSpringDampingRatio", w_WheelJoint_setSpringDampingRatio },
 	{ "getSpringDampingRatio", w_WheelJoint_getSpringDampingRatio },
-	// From Joint.
-	{ "getType", w_Joint_getType },
-	{ "getAnchors", w_Joint_getAnchors },
-	{ "getReactionForce", w_Joint_getReactionForce },
-	{ "getReactionTorque", w_Joint_getReactionTorque },
-	{ "getCollideConnected", w_Joint_getCollideConnected },
-	{ "destroy", w_Joint_destroy },
+	{ "getAxis", w_WheelJoint_getAxis },
 	{ 0, 0 }
 };
 
 extern "C" int luaopen_wheeljoint(lua_State *L)
 {
-	return luax_register_type(L, "WheelJoint", functions);
+	return luax_register_type(L, &WheelJoint::type, w_Joint_functions, w_WheelJoint_functions, nullptr);
 }
 
 } // box2d
