@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -29,7 +29,7 @@ namespace box2d
 
 World *luax_checkworld(lua_State *L, int idx)
 {
-	World *w = luax_checktype<World>(L, idx, "World", PHYSICS_WORLD_T);
+	World *w = luax_checktype<World>(L, idx);
 	if (!w->isValid())
 		luaL_error(L, "Attempt to use destroyed world.");
 	return w;
@@ -39,7 +39,19 @@ int w_World_update(lua_State *L)
 {
 	World *t = luax_checkworld(L, 1);
 	float dt = (float)luaL_checknumber(L, 2);
-	EXCEPT_GUARD(t->update(dt);)
+
+	// Make sure the world callbacks are using the calling Lua thread.
+	t->setCallbacksL(L);
+
+	if (lua_isnoneornil(L, 3))
+		luax_catchexcept(L, [&](){ t->update(dt); });
+	else
+	{
+		int velocityiterations = (int) luaL_checkinteger(L, 3);
+		int positioniterations = (int) luaL_checkinteger(L, 4);
+		luax_catchexcept(L, [&](){ t->update(dt, velocityiterations, positioniterations); });
+	}
+
 	return 0;
 }
 
@@ -92,14 +104,14 @@ int w_World_translateOrigin(lua_State *L)
 	World *t = luax_checkworld(L, 1);
 	float arg1 = (float)luaL_checknumber(L, 2);
 	float arg2 = (float)luaL_checknumber(L, 3);
-	EXCEPT_GUARD(t->translateOrigin(arg1, arg2);)
+	luax_catchexcept(L, [&](){ t->translateOrigin(arg1, arg2); });
 	return 0;
 }
 
 int w_World_setSleepingAllowed(lua_State *L)
 {
 	World *t = luax_checkworld(L, 1);
-	bool b = luax_toboolean(L, 2);
+	bool b = luax_checkboolean(L, 2);
 	t->setSleepingAllowed(b);
 	return 0;
 }
@@ -139,30 +151,30 @@ int w_World_getContactCount(lua_State *L)
 	return 1;
 }
 
-int w_World_getBodyList(lua_State *L)
+int w_World_getBodies(lua_State *L)
 {
 	World *t = luax_checkworld(L, 1);
 	lua_remove(L, 1);
 	int ret = 0;
-	EXCEPT_GUARD(ret = t->getBodyList(L);)
+	luax_catchexcept(L, [&](){ ret = t->getBodies(L); });
 	return ret;
 }
 
-int w_World_getJointList(lua_State *L)
+int w_World_getJoints(lua_State *L)
 {
 	World *t = luax_checkworld(L, 1);
 	lua_remove(L, 1);
 	int ret = 0;
-	EXCEPT_GUARD(ret = t->getJointList(L);)
+	luax_catchexcept(L, [&](){ ret = t->getJoints(L); });
 	return ret;
 }
 
-int w_World_getContactList(lua_State *L)
+int w_World_getContacts(lua_State *L)
 {
 	World *t = luax_checkworld(L, 1);
 	lua_remove(L, 1);
 	int ret = 0;
-	EXCEPT_GUARD(ret = t->getContactList(L);)
+	luax_catchexcept(L, [&](){ ret = t->getContacts(L); });
 	return ret;
 }
 
@@ -178,19 +190,43 @@ int w_World_rayCast(lua_State *L)
 	World *t = luax_checkworld(L, 1);
 	lua_remove(L, 1);
 	int ret = 0;
-	EXCEPT_GUARD(ret = t->rayCast(L);)
+	luax_catchexcept(L, [&](){ ret = t->rayCast(L); });
 	return ret;
 }
 
 int w_World_destroy(lua_State *L)
 {
 	World *t = luax_checkworld(L, 1);
-	EXCEPT_GUARD(t->destroy();)
+	luax_catchexcept(L, [&](){ t->destroy(); });
 	return 0;
 }
 
+int w_World_isDestroyed(lua_State *L)
+{
+	World *w = luax_checktype<World>(L, 1);
+	luax_pushboolean(L, !w->isValid());
+	return 1;
+}
 
-static const luaL_Reg functions[] =
+int w_World_getBodyList(lua_State *L)
+{
+	luax_markdeprecated(L, "World:getBodyList", API_METHOD, DEPRECATED_RENAMED, "World:getBodies");
+	return w_World_getBodies(L);
+}
+
+int w_World_getJointList(lua_State *L)
+{
+	luax_markdeprecated(L, "World:getJointList", API_METHOD, DEPRECATED_RENAMED, "World:getJoints");
+	return w_World_getJoints(L);
+}
+
+int w_World_getContactList(lua_State *L)
+{
+	luax_markdeprecated(L, "World:getContactList", API_METHOD, DEPRECATED_RENAMED, "World:getContacts");
+	return w_World_getContacts(L);
+}
+
+static const luaL_Reg w_World_functions[] =
 {
 	{ "update", w_World_update },
 	{ "setCallbacks", w_World_setCallbacks },
@@ -206,18 +242,25 @@ static const luaL_Reg functions[] =
 	{ "getBodyCount", w_World_getBodyCount },
 	{ "getJointCount", w_World_getJointCount },
 	{ "getContactCount", w_World_getContactCount },
-	{ "getBodyList", w_World_getBodyList },
-	{ "getJointList", w_World_getJointList },
-	{ "getContactList", w_World_getContactList },
+	{ "getBodies", w_World_getBodies },
+	{ "getJoints", w_World_getJoints },
+	{ "getContacts", w_World_getContacts },
 	{ "queryBoundingBox", w_World_queryBoundingBox },
 	{ "rayCast", w_World_rayCast },
 	{ "destroy", w_World_destroy },
+	{ "isDestroyed", w_World_isDestroyed },
+
+	// Deprecated
+	{ "getBodyList", w_World_getBodyList },
+	{ "getJointList", w_World_getJointList },
+	{ "getContactList", w_World_getContactList },
+
 	{ 0, 0 }
 };
 
 extern "C" int luaopen_world(lua_State *L)
 {
-	return luax_register_type(L, "World", functions);
+	return luax_register_type(L, &World::type, w_World_functions, nullptr);
 }
 
 } // box2d
