@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -34,6 +34,8 @@ namespace love
 namespace sound
 {
 
+love::Type SoundData::type("SoundData", &Data::type);
+
 SoundData::SoundData(Decoder *decoder)
 	: data(0)
 	, size(0)
@@ -41,6 +43,9 @@ SoundData::SoundData(Decoder *decoder)
 	, bitDepth(0)
 	, channels(0)
 {
+	if (decoder->getBitDepth() != 8 && decoder->getBitDepth() != 16)
+		throw love::Exception("Invalid bit depth: %d", decoder->getBitDepth());
+
 	size_t bufferSize = 524288; // 0x80000
 	int decoded = decoder->decode();
 
@@ -78,7 +83,7 @@ SoundData::SoundData(Decoder *decoder)
 	if (data && bufferSize > size)
 		data = (uint8 *) realloc(data, size);
 
-	channels = decoder->getChannels();
+	channels = decoder->getChannelCount();
 	bitDepth = decoder->getBitDepth();
 	sampleRate = decoder->getSampleRate();
 }
@@ -103,12 +108,27 @@ SoundData::SoundData(void *d, int samples, int sampleRate, int bitDepth, int cha
 	load(samples, sampleRate, bitDepth, channels, d);
 }
 
+SoundData::SoundData(const SoundData &c)
+	: data(0)
+	, size(0)
+	, sampleRate(0)
+	, bitDepth(0)
+	, channels(0)
+{
+	load(c.getSampleCount(), c.getSampleRate(), c.getBitDepth(), c.getChannelCount(), c.getData());
+}
+
 SoundData::~SoundData()
 {
 	if (data != 0)
 		free(data);
 }
 
+SoundData *SoundData::clone() const
+{
+	return new SoundData(*this);
+}
+
 void SoundData::load(int samples, int sampleRate, int bitDepth, int channels, void *newData)
 {
 	if (samples <= 0)
@@ -117,7 +137,7 @@ void SoundData::load(int samples, int sampleRate, int bitDepth, int channels, vo
 	if (sampleRate <= 0)
 		throw love::Exception("Invalid sample rate: %d", sampleRate);
 
-	if (bitDepth <= 0)
+	if (bitDepth != 8 && bitDepth != 16)
 		throw love::Exception("Invalid bit depth: %d", bitDepth);
 
 	if (channels <= 0)
@@ -159,7 +179,7 @@ size_t SoundData::getSize() const
 	return size;
 }
 
-int SoundData::getChannels() const
+int SoundData::getChannelCount() const
 {
 	return channels;
 }
@@ -176,7 +196,7 @@ int SoundData::getSampleRate() const
 
 int SoundData::getSampleCount() const
 {
-	return (size/channels)/(bitDepth/8);
+	return (int) ((size/channels)/(bitDepth/8));
 }
 
 float SoundData::getDuration() const
@@ -203,6 +223,14 @@ void SoundData::setSample(int i, float sample)
 	}
 }
 
+void SoundData::setSample(int i, int channel, float sample)
+{
+	if (channel < 1 || channel > channels)
+		throw love::Exception("Attempt to set sample from out-of-range channel!");
+
+	return setSample(i * channels + (channel - 1), sample);
+}
+
 float SoundData::getSample(int i) const
 {
 	// Check range.
@@ -222,5 +250,13 @@ float SoundData::getSample(int i) const
 	}
 }
 
+float SoundData::getSample(int i, int channel) const
+{
+	if (channel < 1 || channel > channels)
+		throw love::Exception("Attempt to get sample from out-of-range channel!");
+
+	return getSample(i * channels + (channel - 1));
+}
+
 } // sound
 } // love
