@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2016 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -35,17 +35,21 @@ class SoundData : public love::Data
 {
 public:
 
+	static love::Type type;
+
 	SoundData(Decoder *decoder);
 	SoundData(int samples, int sampleRate, int bitDepth, int channels);
 	SoundData(void *d, int samples, int sampleRate, int bitDepth, int channels);
+	SoundData(const SoundData &c);
 
 	virtual ~SoundData();
 
 	// Implements Data.
+	SoundData *clone() const;
 	void *getData() const;
 	size_t getSize() const;
 
-	virtual int getChannels() const;
+	virtual int getChannelCount() const;
 	virtual int getBitDepth() const;
 	virtual int getSampleRate() const;
 	virtual int getSampleCount() const;
@@ -53,7 +57,9 @@ public:
 	virtual float getDuration() const;
 
 	void setSample(int i, float sample);
+	void setSample(int i, int channel, float sample);
 	float getSample(int i) const;
+	float getSample(int i, int channel) const;
 
 private:
 
