@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2015 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -132,7 +132,7 @@ bool GmeDecoder::isSeekable()
 	return true;
 }
 
-int GmeDecoder::getChannels() const
+int GmeDecoder::getChannelCount() const
 {
 	return 2;
 }
@@ -142,6 +142,11 @@ int GmeDecoder::getBitDepth() const
 	return 16;
 }
 
+double GmeDecoder::getDuration()
+{
+	return -1;
+}
+
 } // lullaby
 } // sound
 } // love
