@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -21,12 +21,20 @@
 #ifndef LOVE_SOUND_LULLABY_MODPLUG_DECODER_H
 #define LOVE_SOUND_LULLABY_MODPLUG_DECODER_H
 
+#include "common/config.h"
+
+#ifndef LOVE_NO_MODPLUG
+
 // LOVE
 #include "common/Data.h"
-#include "Decoder.h"
+#include "sound/Decoder.h"
 
-// SDL_sound
+// libmodplug
+#if defined(LOVE_ANDROID) || defined(LOVE_IOS)
+#include <modplug.h>
+#else
 #include <libmodplug/modplug.h>
+#endif
 
 namespace love
 {
@@ -49,17 +57,23 @@ public:
 	bool seek(float s);
 	bool rewind();
 	bool isSeekable();
-	int getChannels() const;
+	int getChannelCount() const;
 	int getBitDepth() const;
+	double getDuration();
 
 private:
 
 	ModPlugFile *plug;
 	ModPlug_Settings settings;
+
+	double duration;
+
 }; // Decoder
 
 } // lullaby
 } // sound
 } // love
 
+#endif // LOVE_NO_MODPLUG
+
 #endif // LOVE_SOUND_LULLABY_MODPLUG_DECODER_H
