@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -23,10 +23,12 @@
 
 // LOVE
 #include "common/Data.h"
-#include "Decoder.h"
+#include "sound/Decoder.h"
+
+#ifndef LOVE_NOMPG123
 
 // libmpg123
-#ifdef LOVE_MACOSX_USE_FRAMEWORKS
+#ifdef LOVE_APPLE_USE_FRAMEWORKS
 #include <mpg123/mpg123.h>
 #else
 #include <mpg123.h>
@@ -39,6 +41,19 @@ namespace sound
 namespace lullaby
 {
 
+struct DecoderFile
+{
+	unsigned char *data;
+	size_t size;
+	size_t offset;
+
+	DecoderFile(Data *d)
+		: data((unsigned char *) d->getData())
+		, size(d->getSize())
+		, offset(0)
+	{}
+};
+
 class Mpg123Decoder : public Decoder
 {
 public:
@@ -54,25 +69,27 @@ public:
 	bool seek(float s);
 	bool rewind();
 	bool isSeekable();
-	int getChannels() const;
+	int getChannelCount() const;
 	int getBitDepth() const;
+	double getDuration();
 
 private:
 
-	int feed(int bytes);
+	DecoderFile decoder_file;
 
 	mpg123_handle *handle;
 	static bool inited;
 
-	int data_offset;
-	int data_size;
-
 	int channels;
 
+	double duration;
+
 }; // Decoder
 
 } // lullaby
 } // sound
 } // love
 
+#endif // LOVE_NOMPG123
+
 #endif // LOVE_SOUND_LULLABY_LIBMPG123_DECODER_H
