@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2015 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -23,7 +23,8 @@
 
 // LOVE
 #include "common/Data.h"
-#include "Decoder.h"
+#include "common/int.h"
+#include "sound/Decoder.h"
 
 // vorbis
 #define OV_EXCLUDE_STATIC_CALLBACKS
@@ -40,9 +41,9 @@ namespace lullaby
 // Struct for handling data
 struct SOggFile
 {
-	char *dataPtr;	// Pointer to the data in memory
-	int dataSize;	// Size of the data
-	int dataRead;	// How much we've read so far
+	const char *dataPtr;	// Pointer to the data in memory
+	int64 dataSize;	// Size of the data
+	int64 dataRead;	// How much we've read so far
 };
 
 class VorbisDecoder : public Decoder
@@ -59,9 +60,10 @@ public:
 	bool seek(float s);
 	bool rewind();
 	bool isSeekable();
-	int getChannels() const;
+	int getChannelCount() const;
 	int getBitDepth() const;
 	int getSampleRate() const;
+	double getDuration();
 
 private:
 	SOggFile oggFile;				// (see struct)
@@ -70,6 +72,7 @@ private:
 	vorbis_info *vorbisInfo;		// Info
 	vorbis_comment *vorbisComment;	// Comments
 	int endian;						// Endianness
+	double duration;
 }; // VorbisDecoder
 
 } // lullaby
