@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -20,6 +20,8 @@
 
 #include "wrap_Sound.h"
 
+#include "filesystem/wrap_Filesystem.h"
+
 // Implementations.
 #include "lullaby/Sound.h"
 
@@ -28,63 +30,63 @@ namespace love
 namespace sound
 {
 
-static Sound *instance = 0;
+#define instance() (Module::getInstance<Sound>(Module::M_SOUND))
+
+int w_newDecoder(lua_State *L)
+{
+	love::filesystem::FileData *data = love::filesystem::luax_getfiledata(L, 1);
+	int bufferSize = (int) luaL_optinteger(L, 2, Decoder::DEFAULT_BUFFER_SIZE);
+
+	Decoder *t = nullptr;
+	luax_catchexcept(L,
+		[&]() { t = instance()->newDecoder(data, bufferSize); },
+		[&](bool) { data->release(); }
+	);
+
+	if (t == nullptr)
+		return luaL_error(L, "Extension \"%s\" not supported.", data->getExtension().c_str());
+
+	luax_pushtype(L, t);
+	t->release();
+	return 1;
+}
 
 int w_newSoundData(lua_State *L)
 {
-	SoundData *t = 0;
+	SoundData *t = nullptr;
 
 	if (lua_isnumber(L, 1))
 	{
-		int samples = luaL_checkint(L, 1);
-		int sampleRate = luaL_optint(L, 2, Decoder::DEFAULT_SAMPLE_RATE);
-		int bitDepth = luaL_optint(L, 3, Decoder::DEFAULT_BIT_DEPTH);
-		int channels = luaL_optint(L, 4, Decoder::DEFAULT_CHANNELS);
+		int samples = (int) luaL_checkinteger(L, 1);
+		int sampleRate = (int) luaL_optinteger(L, 2, Decoder::DEFAULT_SAMPLE_RATE);
+		int bitDepth = (int) luaL_optinteger(L, 3, Decoder::DEFAULT_BIT_DEPTH);
+		int channels = (int) luaL_optinteger(L, 4, Decoder::DEFAULT_CHANNELS);
 
-		EXCEPT_GUARD(t = instance->newSoundData(samples, sampleRate, bitDepth, channels);)
+		luax_catchexcept(L, [&](){ t = instance()->newSoundData(samples, sampleRate, bitDepth, channels); });
 	}
 	// Must be string or decoder.
 	else
 	{
 		// Convert to Decoder, if necessary.
-		if (!luax_istype(L, 1, SOUND_DECODER_T))
+		if (!luax_istype(L, 1, Decoder::type))
 		{
 			w_newDecoder(L);
 			lua_replace(L, 1);
 		}
 
-		EXCEPT_GUARD(t = instance->newSoundData(luax_checkdecoder(L, 1));)
+		luax_catchexcept(L, [&](){ t = instance()->newSoundData(luax_checkdecoder(L, 1)); });
 	}
 
-	luax_pushtype(L, "SoundData", SOUND_SOUND_DATA_T, t);
-	return 1;
-}
-
-int w_newDecoder(lua_State *L)
-{
-	// Convert to FileData, if necessary.
-	if (lua_isstring(L, 1) || luax_istype(L, 1, FILESYSTEM_FILE_T))
-		luax_convobj(L, 1, "filesystem", "newFileData");
-
-	love::filesystem::FileData *data = luax_checktype<love::filesystem::FileData>(L, 1, "FileData", FILESYSTEM_FILE_DATA_T);
-
-	int bufferSize = luaL_optint(L, 2, Decoder::DEFAULT_BUFFER_SIZE);
-
-	Decoder *t = 0;
-	EXCEPT_GUARD(t = instance->newDecoder(data, bufferSize);)
-
-	if (t == 0)
-		return luaL_error(L, "Extension \"%s\" not supported.", data->getExtension().c_str());
-
-	luax_pushtype(L, "Decoder", SOUND_DECODER_T, t);
+	luax_pushtype(L, t);
+	t->release();
 	return 1;
 }
 
 // List of functions to wrap.
 static const luaL_Reg functions[] =
 {
-	{ "newSoundData",  w_newSoundData },
 	{ "newDecoder",  w_newDecoder },
+	{ "newSoundData",  w_newSoundData },
 	{ 0, 0 }
 };
 
@@ -97,9 +99,10 @@ static const lua_CFunction types[] =
 
 extern "C" int luaopen_love_sound(lua_State *L)
 {
-	if (instance == 0)
+	Sound *instance = instance();
+	if (instance == nullptr)
 	{
-		EXCEPT_GUARD(instance = new lullaby::Sound();)
+		luax_catchexcept(L, [&](){ instance = new lullaby::Sound(); });
 	}
 	else
 		instance->retain();
@@ -108,7 +111,7 @@ extern "C" int luaopen_love_sound(lua_State *L)
 	WrappedModule w;
 	w.module = instance;
 	w.name = "sound";
-	w.flags = MODULE_SOUND_T;
+	w.type = &Sound::type;
 	w.functions = functions;
 	w.types = types;
 
