@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -32,8 +32,6 @@ namespace love
 namespace sound
 {
 
-int w_newSoundData(lua_State *L);
-int w_newDecoder(lua_State *L);
 extern "C" LOVE_EXPORT int luaopen_love_sound(lua_State *L);
 
 } // sound
