@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2015 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -20,22 +20,41 @@
 
 #include "wrap_SoundData.h"
 
-#include "common/wrap_Data.h"
+#include "data/wrap_Data.h"
+
+// Shove the wrap_SoundData.lua code directly into a raw string literal.
+static const char sounddata_lua[] =
+#include "wrap_SoundData.lua"
+;
 
 namespace love
 {
 namespace sound
 {
 
+/**
+ * NOTE: Additional wrapper code is in wrap_SoundData.lua. Be sure to keep it
+ * in sync with any changes made to this file!
+ **/
+
 SoundData *luax_checksounddata(lua_State *L, int idx)
 {
-	return luax_checktype<SoundData>(L, idx, "SoundData", SOUND_SOUND_DATA_T);
+	return luax_checktype<SoundData>(L, idx);
 }
 
-int w_SoundData_getChannels(lua_State *L)
+int w_SoundData_clone(lua_State *L)
+{
+	SoundData *t = luax_checksounddata(L, 1), *c = nullptr;
+	luax_catchexcept(L, [&](){ c = t->clone(); });
+	luax_pushtype(L, c);
+	c->release();
+	return 1;
+}
+
+int w_SoundData_getChannelCount(lua_State *L)
 {
 	SoundData *t = luax_checksounddata(L, 1);
-	lua_pushinteger(L, t->getChannels());
+	lua_pushinteger(L, t->getChannelCount());
 	return 1;
 }
 
@@ -71,9 +90,19 @@ int w_SoundData_setSample(lua_State *L)
 {
 	SoundData *sd = luax_checksounddata(L, 1);
 	int i = (int) luaL_checkinteger(L, 2);
-	float sample = (float) luaL_checknumber(L, 3);
 
-	luax_catchexcept(L, [&](){ sd->setSample(i, sample); });
+	if (lua_gettop(L) > 3)
+	{
+		int channel = luaL_checkinteger(L, 3);
+		float sample = (float) luaL_checknumber(L, 4);
+		luax_catchexcept(L, [&](){ sd->setSample(i, channel, sample); });
+	}
+	else
+	{
+		float sample = (float) luaL_checknumber(L, 3);
+		luax_catchexcept(L, [&](){ sd->setSample(i, sample); });
+	}
+
 	return 0;
 }
 
@@ -82,30 +111,57 @@ int w_SoundData_getSample(lua_State *L)
 	SoundData *sd = luax_checksounddata(L, 1);
 	int i = (int) luaL_checkinteger(L, 2);
 
-	luax_catchexcept(L, [&](){ lua_pushnumber(L, sd->getSample(i)); });
+	if (lua_gettop(L) > 2)
+	{
+		int channel = luaL_checkinteger(L, 3);
+		luax_catchexcept(L, [&](){ lua_pushnumber(L, sd->getSample(i, channel)); });
+	}
+	else
+		luax_catchexcept(L, [&](){ lua_pushnumber(L, sd->getSample(i)); });
 	return 1;
 }
 
-static const luaL_Reg functions[] =
+int w_SoundData_getChannels(lua_State *L)
 {
-	// Data
-	{ "getString", w_Data_getString },
-	{ "getPointer", w_Data_getPointer },
-	{ "getSize", w_Data_getSize },
+	luax_markdeprecated(L, "SoundData:getChannels", API_METHOD, DEPRECATED_RENAMED, "SoundData:getChannelCount");
+	return w_SoundData_getChannelCount(L);
+}
 
-	{ "getChannels", w_SoundData_getChannels },
+static const luaL_Reg w_SoundData_functions[] =
+{
+	{ "clone", w_SoundData_clone },
+	{ "getChannelCount", w_SoundData_getChannelCount },
 	{ "getBitDepth", w_SoundData_getBitDepth },
 	{ "getSampleRate", w_SoundData_getSampleRate },
 	{ "getSampleCount", w_SoundData_getSampleCount },
 	{ "getDuration", w_SoundData_getDuration },
 	{ "setSample", w_SoundData_setSample },
 	{ "getSample", w_SoundData_getSample },
+
+	// Deprecated
+	{ "getChannels", w_SoundData_getChannels },
+
 	{ 0, 0 }
 };
 
 extern "C" int luaopen_sounddata(lua_State *L)
 {
-	return luax_register_type(L, "SoundData", functions);
+	int ret = luax_register_type(L, &SoundData::type, data::w_Data_functions, w_SoundData_functions, nullptr);
+
+	luax_gettypemetatable(L, SoundData::type);
+
+	// Load and execute SoundData.lua, sending the metatable as an argument.
+	if (lua_istable(L, -1))
+	{
+		luaL_loadbuffer(L, sounddata_lua, sizeof(sounddata_lua), "SoundData.lua");
+		lua_pushvalue(L, -2);
+		lua_call(L, 1, 0);
+	}
+
+	// Pop the metatable.
+	lua_pop(L, 1);
+
+	return ret;
 }
 
 } // sound
