@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2015 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -99,6 +99,21 @@ public:
 	 **/
 	virtual bool openURL(const std::string &url) const;
 
+	/**
+	 * Vibrates for the specified amount of seconds.
+	 *
+	 * @param number of seconds to vibrate.
+	 */
+	virtual void vibrate(double seconds) const;
+
+	/**
+	 * Gets if the user is playing music on background.
+	 * Throws an exception on unsupported platforms.
+	 *
+	 * @return Whether a music is playing on background.
+	 **/
+	bool hasBackgroundMusic() const;
+
 	static bool getConstant(const char *in, PowerState &out);
 	static bool getConstant(PowerState in, const char *&out);
 
