@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2015 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -22,7 +22,6 @@
 #define LOVE_THREAD_THREAD_H
 
 // LOVE
-#include "common/runtime.h"
 #include "common/Object.h"
 
 namespace love
@@ -39,7 +38,7 @@ public:
 	virtual void wait() = 0;
 	virtual bool isRunning() = 0;
 
-}; // ThreadObject
+}; // Thread
 
 } // thread
 } // love
