@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2015 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -27,8 +27,7 @@ namespace thread
 
 LuaThread *ThreadModule::newThread(const std::string &name, love::Data *data)
 {
-	LuaThread *lt = new LuaThread(name, data);
-	return lt;
+	return new LuaThread(name, data);
 }
 
 Channel *ThreadModule::newChannel()
