@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -22,7 +22,7 @@
 #define LOVE_THREAD_SDL_THREAD_H
 
 // LOVE
-#include <thread/Thread.h>
+#include "thread/Thread.h"
 #include "threads.h"
 
 // SDL
@@ -34,6 +34,7 @@ namespace thread
 {
 namespace sdl
 {
+
 class Thread : public thread::Thread
 {
 public:
