@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -21,9 +21,13 @@
 #ifndef LOVE_THREAD_THREADS_H
 #define LOVE_THREAD_THREADS_H
 
-#include <common/config.h>
+// LOVE
+#include "common/config.h"
 #include "Thread.h"
 
+// C++
+#include <string>
+
 namespace love
 {
 namespace thread
@@ -53,6 +57,7 @@ class Lock
 public:
 	Lock(Mutex *m);
 	Lock(Mutex &m);
+	Lock(Lock &&other);
 	~Lock();
 
 private:
@@ -72,9 +77,11 @@ private:
 	Mutex *mutex;
 };
 
-class Threadable
+class Threadable : public love::Object
 {
 public:
+	static love::Type type;
+
 	Threadable();
 	virtual ~Threadable();
 
@@ -83,15 +90,50 @@ public:
 	bool start();
 	void wait();
 	bool isRunning() const;
+	const char *getThreadName() const;
 
 protected:
+
 	Thread *owner;
+	std::string threadName;
+
+};
+
+class MutexRef
+{
+public:
+	MutexRef();
+	~MutexRef();
+
+	operator Mutex*() const;
+	Mutex *operator->() const;
+
+private:
+	Mutex *mutex;
+};
+
+class ConditionalRef
+{
+public:
+	ConditionalRef();
+	~ConditionalRef();
+
+	operator Conditional*() const;
+	Conditional *operator->() const;
+
+private:
+	Conditional *conditional;
 };
 
 Mutex *newMutex();
 Conditional *newConditional();
 Thread *newThread(Threadable *t);
 
+#if defined(LOVE_LINUX)
+void disableSignals();
+void reenableSignals();
+#endif
+
 } // thread
 } // love
 
