@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -23,48 +23,46 @@
 // LOVE
 #include "wrap_Timer.h"
 
-#include "sdl/Timer.h"
-
 namespace love
 {
 namespace timer
 {
 
-static Timer *instance = 0;
+#define instance() (Module::getInstance<Timer>(Module::M_TIMER))
 
-int w_step(lua_State *)
+int w_step(lua_State *L)
 {
-	instance->step();
-	return 0;
+	lua_pushnumber(L, instance()->step());
+	return 1;
 }
 
 int w_getDelta(lua_State *L)
 {
-	lua_pushnumber(L, instance->getDelta());
+	lua_pushnumber(L, instance()->getDelta());
 	return 1;
 }
 
 int w_getFPS(lua_State *L)
 {
-	lua_pushinteger(L, instance->getFPS());
+	lua_pushinteger(L, instance()->getFPS());
 	return 1;
 }
 
 int w_getAverageDelta(lua_State *L)
 {
-	lua_pushnumber(L, instance->getAverageDelta());
+	lua_pushnumber(L, instance()->getAverageDelta());
 	return 1;
 }
 
 int w_sleep(lua_State *L)
 {
-	instance->sleep(luaL_checknumber(L, 1));
+	instance()->sleep(luaL_checknumber(L, 1));
 	return 0;
 }
 
 int w_getTime(lua_State *L)
 {
-	lua_pushnumber(L, instance->getTime());
+	lua_pushnumber(L, instance()->getTime());
 	return 1;
 }
 
@@ -83,9 +81,10 @@ static const luaL_Reg functions[] =
 
 extern "C" int luaopen_love_timer(lua_State *L)
 {
-	if (instance == 0)
+	Timer *instance = instance();
+	if (instance == nullptr)
 	{
-		EXCEPT_GUARD(instance = new love::timer::sdl::Timer();)
+		luax_catchexcept(L, [&](){ instance = new love::timer::Timer(); });
 	}
 	else
 		instance->retain();
@@ -93,7 +92,7 @@ extern "C" int luaopen_love_timer(lua_State *L)
 	WrappedModule w;
 	w.module = instance;
 	w.name = "timer";
-	w.flags = MODULE_T;
+	w.type = &Module::type;
 	w.functions = functions;
 	w.types = 0;
 
