@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2018 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -29,25 +29,6 @@ namespace love
 namespace window
 {
 
-int w_getDisplayCount(lua_State *L);
-int w_setMode(lua_State *L);
-int w_getMode(lua_State *L);
-int w_getFullscreenModes(lua_State *L);
-int w_setFullscreen(lua_State *L);
-int w_getFullscreen(lua_State *L);
-int w_isCreated(lua_State *L);
-int w_getWidth(lua_State *L);
-int w_getHeight(lua_State *L);
-int w_getDimensions(lua_State *L);
-int w_getDesktopDimensions(lua_State *L);
-int w_setIcon(lua_State *L);
-int w_getIcon(lua_State *L);
-int w_setTitle(lua_State *L);
-int w_getTitle(lua_State *L);
-int w_hasFocus(lua_State *L);
-int w_hasMouseFocus(lua_State *L);
-int w_isVisible(lua_State *L);
-int w_getPixelScale(lua_State *L);
 extern "C" LOVE_EXPORT int luaopen_love_window(lua_State *L);
 
 } // window
